var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"FBX\">\n      <meta name=\"release\" content=\"2018\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2017-04-25\">\n      <meta name=\"topicid\" content=\"GUID-0D483705-23D9-476D-A567-09609396B190\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Instancing - Sharing a Mesh</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-0D483705-23D9-476D-A567-09609396B190\"></a><div class=\"head\">\n            <h1>Instancing - Sharing a Mesh</h1>\n         </div>\n         <p><a name=\"GUID-9AB1A27F-7C35-4FD6-81F6-48C2B0209CC8\"></a>To reduce memory requirements, a single instance of <span class=\"code\">FbxMesh</span> can be bound to multiple instances of FbxNode. Imagine that you need a program where\n            all cubes looked alike, but you needed many thousands of them. You could save memory\n            by creating one <span class=\"code\">FbxMesh</span> object when the program starts up. Then, every time you needed a new cube, you create\n            a new <span class=\"code\">FbxNode</span> object, then point that node to the one mesh. This is called <em class=\"mild\">instancing</em>. \n         </p>\n         <p><a name=\"WS73099CC142F487551FEA285E1221E4F9FF8-7F44\"></a> In general, you can save memory by having many node objects share one node attribute\n            object (i.e., one object of any subclass of <span class=\"code\">FbxNodeAttribute</span>). The following function illustrates how to bind a <span class=\"code\">FbxMesh</span> to a newly created node. \n         </p>\n         <div class=\"codeBlock\"><pre>// Create a cube instance with the given mesh as node attribute, and add it to the scene.\nFbxNode* CreateCubeInstance(FbxScene* pScene, const char* pName, FbxMesh* pFirstCube)\n{\n    // create a FbxNode\n    FbxNode* lNode = FbxNode::Create(pScene,pName);\n\n    // set the node attribute\n    lNode-&gt;SetNodeAttribute(pFirstCube);\n\n    // rescale the cube\n    lNode-&gt;LclScaling.Set(FbxVector4(0.3, 0.3, 0.3));\n\n        // Add node to the scene\n    pScene-&gt;GetRootNode()-&gt;AddChild(lNode);\n\n    // return the FbxNode\n    return lNode;\n}</pre></div>\n         <p><a name=\"GUID-15D33FEE-343C-4F12-9875-0A3C6052C8D2\"></a>Each node is an <em class=\"mild\">instance</em> of the mesh, NURBS, or other scene element. If you export your scene to an FBX file,\n            instancing also reduces the file size. You can also save memory by having multiple\n            nodes share textures, materials, animation curves, etc. \n         </p>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";