var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"FBX\">\n      <meta name=\"release\" content=\"2018\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2017-04-25\">\n      <meta name=\"topicid\" content=\"GUID-1870771B-1A50-4B8C-B2A0-6979509DE0C1\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Evaluating the Animation in a Scene</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-1870771B-1A50-4B8C-B2A0-6979509DE0C1\"></a><div class=\"head\">\n            <h1>Evaluating the Animation in a Scene</h1>\n         </div>\n         <p><a name=\"GUID-EEFA3FD9-051D-43ED-9F34-2AF98090F80C\"></a>To render an animated scene (<span class=\"code\">FbxScene</span>), you must evaluate the animated properties of objects (<span class=\"code\">FbxObject</span>) in a scene at different points in time. For example, to display a moving race car\n            that is represented by a mesh, you must animate the local translation property of\n            the node whose node attribute is the mesh. See \n            		<span class=\"code\">FbxNode::LclTranslation</span>. \n            	 \n         </p><a name=\"OL_8860F267FA3449CEA03DE7721528D103\"></a><p class=\"list-intro\"><a name=\"GUID-494D53BC-BD0E-4E5F-AD78-1BEFCC316251\"></a>Perform the following steps to evaluate the animation in a scene:\n         </p>\n         <ol type=\"1\" start=\"1\">\n            <li>Get an evaluator object from the scene object that contains the node. \n               		  \n               <div class=\"codeBlock\"><pre>\n// Let us assume that myScene is already created\nFbxAnimEvaluator* mySceneEvaluator = MyScene-&gt;GetAnimationEvaluator();\n</pre></div> \n            </li>\n            <li>Create a time object. \n               		  \n               <div class=\"codeBlock\"><pre>\nFbxTime myTime;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;// The time for each key in the animation curve(s)\nmyTime.SetSecondDouble(0.0);&nbsp;&nbsp;&nbsp;// Starting time\n</pre></div> \n            </li>\n            <li>Create a node object. \n               		  \n               <div class=\"codeBlock\"><pre>\n// Create a node object\nFbxNode* myMeshNode = FbxNode::Create (myScene, \"\");\n// ... Code to connect myMeshNode to a mesh object\n</pre></div> \n            </li>\n            <li>Get the global transformation matrix or local transformation matrix of the node at\n               a specific time. \n               		  <a name=\"UL_283CDBB4DDC145A59A87052C0AB87D40\"></a><ul>\n                  <li>To get the global transformation matrix: \n                     				\n                     <div class=\"codeBlock\"><pre>\n// Get a reference to node’s global transform.\nFbxMatrix&amp; GlobalMatrix = mySceneEvaluator-&gt;GetNodeGlobalTransform(myMeshNode, myTime);\n</pre></div> \n                  </li>\n                  <li>To get the local transformation matrix: \n                     				\n                     <div class=\"codeBlock\"><pre>\n// Get a reference to node’s local transform.\nFbxMatrix&amp; GlobalMatrix = mySceneEvaluator-&gt;GetNodeLocalTransform(myMeshNode, myTime);\n</pre></div> \n                  </li>\n               </ul> \n            </li>\n            <li> Evaluate the camera’s position expressed as a vector at a specific time. \n               		  \n               <div><a name=\"GUID-006A7FC6-6350-4D3B-842F-E1A1EC3B3567\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span>A transformation matrix is not needed to evaluate a camera\'s position. \n                     		  \n                  </div>\n               </div> \n               <div class=\"codeBlock\"><pre>\n// Given a scene, we can create a camera\nFbxCamera* myCamera = FbxCamera::Create (myScene, \"\");\n&nbsp;\n// Store the value of the property in an animation curve node\nFbxAnimCurveNode&amp;  myCameraPositionVector;\n&nbsp;\n// Get and store the value of the camera\'s local translation\nmyCameraPositionVector = mySceneEvaluator-&gt;GetPropertyValue (myCamera-&gt;Position, myTime);\n</pre></div> \n            </li>\n            <li>Evaluate a material property at a specific time.\n               		  \n               <div class=\"codeBlock\"><pre>\nFbxTime time;\ntime.SetSecondDouble(3.5);\nFbxColor color = MyMaterial-&gt;GetDiffuseColor()-&gt;EvaluateValue(time);\n</pre></div> \n            </li>\n         </ol>\n         <p><a name=\"GUID-CDE02936-F588-4938-A0F5-96001763317C\"></a>See \n            		<span class=\"code\">Animation/main.cxx</span> for an example that illustrates the use of animation stacks, animation layers, animation\n            curve nodes, and animation curves. \n            	 \n         </p>\n         <div class=\"section\"><a name=\"SECTION_8A1D5FE711654452A64001F8654C995B\"></a> \n            <h2><a name=\"GUID-208C905C-2522-43E6-8005-ED8AC17DDA3E\"></a>Using the Animation Curve Node to Store the Value of a FBX Property\n            </h2> \n            <p><a name=\"GUID-CC6D77A7-60EB-4EA3-8A9E-586B58D62E51\"></a>In the code snippet above, the \n               		  <span class=\"code\">FbxAnimCurveNode</span> object is not used as a connection point between an animation curve and a FBX property.\n               Instead, \n               		  <span class=\"code\">FbxAnimCurveNode</span> is used as a convenient place to store the vector of the property returned by the\n               evaluator. The animatable properties of FBX objects come in many data types. Some\n               data types are scalars, for example, \n               		  <span class=\"code\">FbxDouble1</span>, \n               		  <span class=\"code\">FbxInteger1</span>, and \n               		  <span class=\"code\">FbxBool1</span>. Other data types have triplet (X, Y, Z) values that are stored as vectors. \n               		\n            </p> \n            <p><a name=\"WS1A9193826455F5FF4E544F0312754F1F577-7B65\"></a><span class=\"code\">FbxNode::LclTranslation</span> is of type \n               		  <span class=\"code\">FbxDouble3</span>, which is a vector with three elements. \n               		  <span class=\"code\">FbxAnimCurveNode</span> is a container that can store the value of any FBX property irrespective of the data\n               type. The member functions of \n               		  <span class=\"code\">FbxAnimCurveNode</span> enables you to access the value of each element in a vector, for example, to get\n               the value of channel X, the value of channel Y, and the value of channel Z. \n               		\n            </p> \n         </div>\n         <div class=\"see-also\">\n            <h4>See Also</h4>\n            <div class=\"related-links\">\n               <ul>\n                  <li> \n                     <p><a name=\"WS1A9193826455F5FF4E544F0312754F1F577-7B64\"></a> <span class=\"code\">FbxNode::LclTranslation</span> \n                     </p> \n                  </li>\n                  <li> \n                     <p><a name=\"WS1A9193826455F5FF4E544F0312754F1F577-7B63\"></a> <span class=\"code\">FbxAnimCurveNode::GetChannelCount</span>, \n                        			 <span class=\"code\">FbxAnimCurveNode::GetChannelValue</span>, and related functions. \n                        		  \n                     </p> \n                  </li>\n                  <li> \n                     <p><a name=\"WS73099CC142F4875516078EA312754F1D2D4-7C28\"></a> <span class=\"code\">fbxdatatypes.h</span>, for a full list of FBX data types. \n                        		  \n                     </p> \n                  </li>\n               </ul>\n            </div>\n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";