var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"FBX\">\n      <meta name=\"release\" content=\"2018\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2017-04-25\">\n      <meta name=\"topicid\" content=\"GUID-3E0DCCD0-5F9A-44D0-8D5E-423237D49DB6\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Materials</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-3E0DCCD0-5F9A-44D0-8D5E-423237D49DB6\"></a><div class=\"head\">\n            <h1>Materials</h1>\n         </div>\n         <div><a name=\"GUID-47F815E0-FE0A-4ED9-A41A-9716E48A91CD\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span>The sample code in this topic is also presented in the <span class=\"code\">ExportScene03/main.cxx</span> sample program. \n            </div>\n         </div>\n         <div class=\"section\"><a name=\"SECTION_A4E401EAD97446EDA93232D7C4D3DADE\"></a> \n            <h2><a name=\"GUID-D6B3A73D-BE29-4E93-AF95-7E8B847AC146\"></a>Creating a Material\n            </h2> \n            <p><a name=\"GUID-7485E7C2-7831-434C-952A-6A82660CA735\"></a> The <span class=\"code\">FbxSurfaceMaterial</span> class is the base class for Lambertian (<span class=\"code\">FbxSurfaceLambert</span>) and Phong (<span class=\"code\">FbxSurfacePhong</span>) materials. In the following code sample, we create five materials and add them to\n               the <span class=\"code\">FbxNode</span> of a <span class=\"code\">FbxMesh</span> via <span class=\"code\">FbxNode::AddMaterial()</span>. Once materials are bound to a node, they can be mapped to the polygons of a mesh.\n               \n            </p> \n            <div class=\"codeBlock\"><pre>// Create materials for pyramid.\nvoid CreateMaterials(FbxScene* pScene, FbxMesh* pMesh)\n{\n    int i;\n\n    for (i = 0; i &lt; 5; i++ )\n    {\n        FbxString lMaterialName = \"material\";\n        FbxString lShadingName = \"Phong\";\n        lMaterialName += i;\n        FbxDouble3 lBlack(0.0, 0.0, 0.0);\n        FbxDouble3 lRed(1.0, 0.0, 0.0);\n        FbxDouble3 lColor;\n        FbxSurfacePhong *lMaterial = FbxSurfacePhong::Create(pScene, lMaterialName.Buffer());\n\n\n        // Generate primary and secondary colors.\n        lMaterial-&gt;Emissive.Set(lBlack);\n        lMaterial-&gt;Ambient.Set(lRed);\n        lColor = FbxDouble3(i &gt; 2   ? 1.0 : 0.0, \n            i &gt; 0 &amp;&amp; i &lt; 4 ? 1.0 : 0.0, \n            i % 2   ? 0.0 : 1.0);\n        lMaterial-&gt;Diffuse.Set(lColor);\n        lMaterial-&gt;TransparencyFactor.Set(0.0);\n        lMaterial-&gt;ShadingModel.Set(lShadingName);\n        lMaterial-&gt;Shininess.Set(0.5);\n\n        //get the node of mesh, add material for it.\n        FbxNode* lNode = pMesh-&gt;GetNode();\n        if(lNode)             \n            lNode-&gt;AddMaterial(lMaterial);\n    }  \n}\n</pre></div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_D760C05E887242C28FEE54DB069525EB\"></a> \n            <h2><a name=\"GUID-EEABA004-5886-4B3F-8A15-5FBD3F4FCA54\"></a>Example: Creating a Square Pyramid with Materials\n            </h2> \n            <p><a name=\"GUID-B6DA1889-5928-435D-A997-4FBE88C89F5E\"></a>The following code sample uses the <span class=\"code\">CreateMaterials()</span> function defined above to bind five materials to the five faces of a square pyramid.\n               We first begin by defining the control points and the normals of the pyramid. \n            </p> \n            <div class=\"codeBlock\"><pre>// Create a pyramid with materials. \nFbxNode* CreatePyramidWithMaterials(FbxScene* pScene, char* pName)\n{\n    int i, j;\n    FbxMesh* lMesh = FbxMesh::Create(pScene, pName);\n\n    FbxVector4 vertex0(-50, 0, 50);\n    FbxVector4 vertex1(50, 0, 50);\n    FbxVector4 vertex2(50, 0, -50);\n    FbxVector4 vertex3(-50, 0, -50);\n    FbxVector4 vertex4(0, 100, 0);\n\n    FbxVector4 lNormalP0(0, 1, 0);\n    FbxVector4 lNormalP1(0, 0.447, 0.894);\n    FbxVector4 lNormalP2(0.894, 0.447, 0);\n    FbxVector4 lNormalP3(0, 0.447, -0.894);\n    FbxVector4 lNormalP4(-0.894, 0.447, 0);\n\n    // Create control points.\n    lMesh-&gt;InitControlPoints(16);\n    FbxVector4* lControlPoints = lMesh-&gt;GetControlPoints();\n\n    lControlPoints[0] = vertex0;\n    lControlPoints[1] = vertex1;\n    lControlPoints[2] = vertex2;\n    lControlPoints[3] = vertex3;\n    lControlPoints[4] = vertex0;\n    lControlPoints[5] = vertex1;\n    lControlPoints[6] = vertex4;\n    lControlPoints[7] = vertex1;\n    lControlPoints[8] = vertex2;\n    lControlPoints[9] = vertex4;\n    lControlPoints[10] = vertex2;\n    lControlPoints[11] = vertex3;\n    lControlPoints[12] = vertex4;\n    lControlPoints[13] = vertex3;\n    lControlPoints[14] = vertex0;\n    lControlPoints[15] = vertex4;\n\n    // specify normals per control point.\n\n    FbxGeometryElementNormal* lNormalElement= lMesh-&gt;CreateElementNormal();\n    lNormalElement-&gt;SetMappingMode(FbxGeometryElement::eByControlPoint);\n    lNormalElement-&gt;SetReferenceMode(FbxGeometryElement::eDirect);\n\n    lNormalElement-&gt;GetDirectArray().Add(lNormalP0);\n    lNormalElement-&gt;GetDirectArray().Add(lNormalP0);\n    lNormalElement-&gt;GetDirectArray().Add(lNormalP0);\n    lNormalElement-&gt;GetDirectArray().Add(lNormalP0);\n    lNormalElement-&gt;GetDirectArray().Add(lNormalP1);\n    lNormalElement-&gt;GetDirectArray().Add(lNormalP1);\n    lNormalElement-&gt;GetDirectArray().Add(lNormalP1);\n    lNormalElement-&gt;GetDirectArray().Add(lNormalP2);\n    lNormalElement-&gt;GetDirectArray().Add(lNormalP2);\n    lNormalElement-&gt;GetDirectArray().Add(lNormalP2);\n    lNormalElement-&gt;GetDirectArray().Add(lNormalP3);\n    lNormalElement-&gt;GetDirectArray().Add(lNormalP3);\n    lNormalElement-&gt;GetDirectArray().Add(lNormalP3);\n    lNormalElement-&gt;GetDirectArray().Add(lNormalP4);\n    lNormalElement-&gt;GetDirectArray().Add(lNormalP4);\n    lNormalElement-&gt;GetDirectArray().Add(lNormalP4);\n</pre></div> \n            <p><a name=\"GUID-40EC32C2-9560-4151-A48A-099F69ABADE3\"></a>We then create a new material layer element (<span class=\"code\">FbxGeometryElementMaterial</span>) within the mesh to specify that the materials are bound to the <em class=\"strong\">control points</em> of the polygons in the mesh. The function <span class=\"code\">FbxMesh::BeginPolygon()</span> uses an index to determine which material will be bound to the new polygon. This\n               index refers to the position of the material stored in the <span class=\"code\">FbxNode</span>. \n            </p> \n            <div class=\"codeBlock\"><pre>// Array of polygon vertices.\n    int lPolygonVertices[] = { 0, 3, 2, 1,\n        4, 5, 6,\n        7, 8, 9,\n        10, 11, 12,\n        13, 14, 15 };\n\n    // Set material mapping.\n    FbxGeometryElementMaterial* lMaterialElement = lMesh-&gt;CreateElementMaterial();\n    lMaterialElement-&gt;SetMappingMode(FbxGeometryElement::eByPolygon);\n    lMaterialElement-&gt;SetReferenceMode(FbxGeometryElement::eIndexToDirect);\n\n    // Create polygons. Assign material indices.\n\n    // Pyramid base.\n    lMesh-&gt;BeginPolygon(0); // Material index.\n\n    for(j = 0; j &lt; 4; j++)\n    {\n        lMesh-&gt;AddPolygon(lPolygonVertices[j]); // Control point index.\n    }\n\n    lMesh-&gt;EndPolygon ();\n\n    // Pyramid sides.\n    for(i = 1; i &lt; 5; i++)\n    {\n        lMesh-&gt;BeginPolygon(i); // Material index.\n\n        for(j = 0; j &lt; 3; j++)\n        {\n            lMesh-&gt;AddPolygon(lPolygonVertices[4 + 3*(i - 1) + j]); // Control point index.\n        }\n\n        lMesh-&gt;EndPolygon ();\n    }\n\n\n    FbxNode* lNode = FbxNode::Create(pScene,pName);\n\n    lNode-&gt;SetNodeAttribute(lMesh);\n\n    CreateMaterials(pScene, lMesh);\n\n    return lNode;\n}</pre></div> \n         </div><span class=\"noindex\">\n            <div class=\"topic-list\">\n               <h4 class=\"related\">Topics in this section</h4>\n               <ul class=\"jumplist\">\n                  <li class=\"topiclist-litem\"><a href=\"GUID-78414234-8644-4725-9FAF-51F2175D3D2D.htm\">Using hardware shaders to create materials</a></li>\n               </ul>\n            </div></span><div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";