var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"FBX\">\n      <meta name=\"release\" content=\"2018\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2017-04-25\">\n      <meta name=\"topicid\" content=\"GUID-452768B5-7A4F-45BD-AC37-DC1A034DCF3B\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Layered Textures</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-452768B5-7A4F-45BD-AC37-DC1A034DCF3B\"></a><div class=\"head\">\n            <h1>Layered Textures</h1>\n         </div>\n         <p><a name=\"GUID-235016F3-18F2-440B-B9EA-0BD486128C38\"></a>In this tutorial topic, we present the use of <span class=\"code\">FbxLayeredTexture</span> to layer four instances of <span class=\"code\">FbxFileTexture</span>. We also expand on the concept of <span class=\"code\">FbxLayerElement</span> to define a mesh\'s normals and UV coordinates. The code in this topic is written\n            in Python, which applies directly to C++. \n         </p>\n         <div class=\"section\"><a name=\"SECTION_F4483F0B374C46F991B54C6846DF0CF3\"></a> \n            <h2><a name=\"GUID-D2117DAB-F389-454E-8B2E-CC30DE62A1AB\"></a>Example: Layering Four Textures\n            </h2> \n            <p><a name=\"GUID-C1170F52-6A43-4D00-984E-38A0B0EDE899\"></a><em class=\"strong\">Required Resources:</em> from left to right: \"<span class=\"code\">one.jpg</span>\", \"<span class=\"code\">two.jpg</span>\", \"<span class=\"code\">three.jpg</span>\", \"<span class=\"code\">four.jpg</span>\". Observe that the width and height of these files are multiples of two (128x128),\n               which is recommended for texture dimensions. \n            </p> \n            <p><a name=\"GUID-ACA3D5F7-10B7-4E77-AB52-6DD55DFE2736\"></a></p> \n            <p><a name=\"GUID-D076E149-B703-4796-A214-C6060B07FC45\"></a> <img src=\"../images/GUID-3D0297FC-DC46-47A6-81DC-257D4218AEAC-low.jpg\"> <img src=\"../images/GUID-2F222C59-D5C3-4ED7-80DE-6460F8517C34-low.jpg\"> <img src=\"../images/GUID-21D5C1AF-658E-4F36-B60E-6DF8B9E00649-low.jpg\"> <img src=\"../images/GUID-A09BAEED-D6BC-432D-AC55-A49DD637340D-low.jpg\"> \n            </p> \n            <p><a name=\"GUID-F9F25F6F-A6AF-4269-B252-1305704E0185\"></a><em class=\"strong\">Sample Output:</em> This screenshot was taken after running the Python program, and importing the resulting\n               <span class=\"code\">.fbx</span> file in Autodesk MotionBuilder. \n            </p> \n            <p><a name=\"GUID-E8461807-14A2-4579-B0F6-4D149B2E61FE\"></a></p> <img src=\"../images/GUID-31762A07-3AA9-404B-8EC5-82166303CF50-low.png\"> \n            <p><a name=\"GUID-5C4A5DEE-05BC-4BD6-B3BF-9B0589DA9C04\"></a><em class=\"strong\">Program Summary:</em> The Python program below creates a scene and exports it to a <span class=\"code\">.fbx</span> file. This scene contains a planar mesh textured using an instance of <span class=\"code\">FbxLayeredTexture</span>. This instance of <span class=\"code\">FbxLayeredTexture</span> is composed of four additively blended instances of <span class=\"code\">FbxFileTexture</span>. The <span class=\"code\">FbxLayeredTexture</span> is connected to the diffuse property of an instance of <span class=\"code\">FbxSurfaceLambert</span>. As illustrated by the following diagram, this instance of <span class=\"code\">FbxSurfaceLambert</span> is added to the <span class=\"code\">FbxNode</span>\'s list of materials. \n            </p> \n            <p><a name=\"GUID-7D2181CC-4AA1-4796-8EE6-F98DA9CB8E69\"></a></p> <img src=\"../images/GUID-A392F017-D4A7-4538-AF02-09845AE13211-low.png\"> \n            <p><a name=\"GUID-9FF086BB-FD1C-4F5A-BA37-1116AB49BD3E\"></a>The planar <span class=\"code\">FbxMesh</span> is set as the <span class=\"code\">FbxNode</span>\'s attribute. The normal values and UV coordinates for this mesh are set within its\n               0th <span class=\"code\">FbxLayer</span> using <span class=\"code\">FbxLayerElementNormal</span> and <span class=\"code\">FbxLayerElementUV</span>. The material is bound to the planar polygon when <span class=\"code\">FbxMesh.BeginPolygon()</span> is called with \'0\' as a parameter. This 0 refers to the (only) material contained\n               in the parent <span class=\"code\">FbxNode</span>\'s list of materials, i.e. <span class=\"code\">FbxSurfaceLambert</span> mentioned previously. \n            </p> \n            <div><a name=\"GUID-FBBA9102-C1C7-4A0A-8DBB-DFDFC8B2D04A\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span>To run this Python program successfully, ensure that the four <span class=\"code\">.jpg</span> images above are in the same folder as this Python program when it is executed. \n               </div>\n            </div> \n            <div class=\"codeBlock\"><pre>\'\'\'\nlayeredTextures.py\n\n    &gt; creates a plane in the scene whose four textures are layered \n      one on top of another.\n\n    &gt; requires texture files: one.jpg, two.jpg, three.jpg, four.jpg\n\'\'\'\n\nimport fbx\nimport FbxCommon\n\nvertices = [fbx.FbxVector4( -5, -5,  0 ), # 0 - vertex index.\n            fbx.FbxVector4(  5, -5,  0 ), # 1\n            fbx.FbxVector4(  5,  5,  0 ), # 2\n            fbx.FbxVector4( -5,  5,  0 )] # 3\n\nnormalPosZ = fbx.FbxVector4( 0, 0, 1 ) # positive-Z normal vector. \n\n# Define the filename and alpha transparency of each texture.\ntextureFilenames = [ ( \'one.jpg\',   0.2 ), \n                     ( \'two.jpg\',   0.5 ), \n                     ( \'three.jpg\', 0.4 ),\n                     ( \'four.jpg\',  1.0 ) ]\n\nsaveFilename = \'layeredTextures.fbx\'\n\n###############################################################\n# Helper Function(s).                                         #\n###############################################################\ndef createPlane(pScene):\n    \'\'\' Create a planar polygon as a child to the root node. \'\'\'\n    \n    rootNode = pScene.GetRootNode()\n    \n    #======================================================\n    # Node definition and mesh object creation. \n    #======================================================\n    # Create the node.\n    newNode = fbx.FbxNode.Create( pScene, \'planeNode\' )\n    rootNode.AddChild( newNode )\n    \n    # Create the mesh node attribute.\n    newMesh = fbx.FbxMesh.Create( pScene, \'planeMesh\' )\n    newNode.SetNodeAttribute( newMesh )\n    \n    # Create a Lambertian material and assign it to the mesh\'s node.\n    applyLambertMaterial( pScene, newNode )\n    \n    # Create and apply the layered texture.\n    applyLayeredTexture( pScene, newNode )\n    \n    # Set the node\'s shading mode to view textures.\n    newNode.SetShadingMode( fbx.FbxNode.eTextureShading )\n    \n    #======================================================\n    # Control points.\n    #======================================================\n    # Define the vertices of the polygon.\n    global vertices\n    newMesh.InitControlPoints( 4 )\n    newMesh.SetControlPointAt( vertices[0], 0 )\n    newMesh.SetControlPointAt( vertices[1], 1 )\n    newMesh.SetControlPointAt( vertices[2], 2 )\n    newMesh.SetControlPointAt( vertices[3], 3 )\n    \n    #======================================================\n    # Normals.\n    #======================================================\n    # For normals and UV coordinates, we will be using the\n    # Layer indexed at 0. If it doesn\'t exist, we will create it.\n    layer = newMesh.GetLayer( 0 )\n    if( not layer ):\n        newMesh.CreateLayer()\n        layer = newMesh.GetLayer( 0 )\n    \n    # Create a normal layer element.\n    normalLayerElement = fbx.FbxLayerElementNormal.Create( newMesh, \'normals\' )\n    \n    # We want to have one normal for each vertex (or control point),    # so we set the mapping mode to eByControlPoint\n    normalLayerElement.SetMappingMode( fbx.FbxLayerElement.eByControlPoint )\n    \n    # Set the normal values for every control point.\n    normalLayerElement.SetReferenceMode( fbx.FbxLayerElement.eDirect )\n    \n    global normalPosZ # positive-Z normals.\n    normalLayerElement.GetDirectArray().Add( normalPosZ )\n    normalLayerElement.GetDirectArray().Add( normalPosZ )\n    normalLayerElement.GetDirectArray().Add( normalPosZ )\n    normalLayerElement.GetDirectArray().Add( normalPosZ )\n    \n    # Assign the normal layer element to the mesh\'s layer 0.\n    layer.SetNormals( normalLayerElement )\n    \n    #======================================================\n    # Diffuse Channel UV Coordinates.\n    #======================================================\n    # Given that we are only dealing with one polygon (a square plane),\n    # we can simplify the code and map the uv coordinates directly to the\n    # four polygon control points.\n    uvDiffuseLayerElement = fbx.FbxLayerElementUV.Create( newMesh, \'diffuseUV\' )\n    uvDiffuseLayerElement.SetMappingMode( fbx.FbxLayerElement.eByPolygonVertex )\n    uvDiffuseLayerElement.SetReferenceMode( fbx.FbxLayerElement.eDirect )\n    \n    # Populate the direct array.\n    uv0 = fbx.FbxVector2( 0, 0 )\n    uv1 = fbx.FbxVector2( 1, 0 )\n    uv2 = fbx.FbxVector2( 1, 1 )\n    uv3 = fbx.FbxVector2( 0, 1 )\n    uvDiffuseLayerElement.GetDirectArray().Add( uv0 )\n    uvDiffuseLayerElement.GetDirectArray().Add( uv1 )\n    uvDiffuseLayerElement.GetDirectArray().Add( uv2 )\n    uvDiffuseLayerElement.GetDirectArray().Add( uv3 )\n    \n    # Assign the uv layer element to the mesh\'s layer 0.\n    layer.SetUVs( uvDiffuseLayerElement, fbx.FbxLayerElement.eTextureDiffuse )\n    \n    #======================================================\n    # Polygon Definition.\n    #======================================================\n    # Create one planar polygon. \n    #    &gt; The 0 in FbxMesh.BeginPolygon() refers to the index of the material to be \n    #      applied to the polygon. Since we only have one material applied to our FbxNode, \n    #      we use index 0.\n    newMesh.BeginPolygon( 0 )\n    for i in range( 0, 4 ):\n        newMesh.AddPolygon( i )\n    newMesh.EndPolygon()\n    \n    return newNode\n\ndef applyLambertMaterial( pScene, pNode ):\n    \'\'\' Apply a Lambertian material to the node. \'\'\'\n        \n    black = fbx.FbxDouble3( 0, 0, 0 )\n    white = fbx.FbxDouble3( 1, 1, 1 )\n    \n    material = fbx.FbxSurfaceLambert.Create( pScene, \'myMaterial\' )\n    material.ShadingModel.Set( \'Lambert\' ) # could also use \'Phong\' if we were using an instance of FbxSurfacePhong\n    material.Emissive.Set( black )\n    material.Ambient.Set( white )\n    material.Diffuse.Set( white )\n    material.TransparencyFactor.Set( 0 )\n    \n    pNode.AddMaterial( material )\n    \n    \ndef applyLayeredTexture( pScene, pNode ):\n    \'\'\' Apply a layered texture on the node using the given texture filenames. \'\'\' \n    global textureFilenames\n    \n    textures = []\n    \n    # Create the individual texture objects to be layered afterwards.\n    for i in range( 0, len( textureFilenames ) ):\n        \n        textureFilename = textureFilenames[i][0]\n        textureAlpha = textureFilenames[i][1]\n        \n        newTexture = fbx.FbxFileTexture.Create( pScene, \'myFileTexture_\' +  str( i ) )\n        newTexture.SetFileName( textureFilename )\n        \n        newTexture.SetTextureUse( fbx.FbxTexture.eStandard )\n        newTexture.SetMappingType( fbx.FbxTexture.eUV )\n        newTexture.SetMaterialUse( fbx.FbxFileTexture.eModelMaterial )\n        newTexture.SetSwapUV( False )\n        newTexture.SetTranslation( 0.0, 0.0 )\n        newTexture.SetScale( 1.0, 1.0 )\n        newTexture.SetRotation( 0.0, 0.0 )\n        newTexture.Alpha.Set( textureAlpha )\n        \n        textures.append( newTexture )\n        \n    # Create the layered texture object, and layer the textures within it.\n    layeredTexture = fbx.FbxLayeredTexture.Create( pScene, \'myLayeredTexture\' )\n     \n    for i in range( 0, len( textures ) ):\n        texture = textures[ i ]\n        layeredTexture.ConnectSrcObject( texture )\n        layeredTexture.SetTextureBlendMode( i, fbx.FbxLayeredTexture.eAdditive )\n    \n    # Connect the layered texture to the node\'s material indexed at 0.\n    # The layered texture is connected on the material\'s diffuse channel. \n    material = pNode.GetMaterial( 0 )\n    material.Diffuse.ConnectSrcObject( layeredTexture )\n\n\n###############################################################\n# Main.                                                       #\n###############################################################\ndef main():\n    \'\'\' Main Program. \'\'\'\n    fbxManager = fbx.FbxManager.Create()\n    fbxScene = fbx.FbxScene.Create( fbxManager, \'\' )\n    \n    # Create the textured planar mesh.\n    newNode = createPlane( fbxScene )\n\n    # Save the file.\n    global saveFilename\n    FbxCommon.SaveScene( fbxManager, fbxScene, saveFilename, pEmbedMedia=True )\n    \n    # Clean up.\n    fbxManager.Destroy()\n    del fbxManager, fbxScene, newNode\n    \nif __name__ == \'__main__\':\n    main()</pre></div> \n         </div>\n         <div class=\"see-also\">\n            <h4>See Also</h4>\n            <div class=\"related-links\">\n               <ul>\n                  <li> \n                     <p><a name=\"WS73099CC142F487551FEA285E1221E4F9FF8-3BC3\"></a> <span class=\"code\">FbxLayerElementUV</span>, <span class=\"code\">FbxLayerElementNormal</span> - UV and Normal definitions \n                     </p> \n                  </li>\n                  <li> \n                     <p><a name=\"WS73099CC142F487551FEA285E1221E4F9FF8-3BC2\"></a> <span class=\"code\">FbxLayerElement::EMappingMode</span> - An enum that specifies how the layer element is mapped to the surface. \n                     </p> \n                  </li>\n                  <li> \n                     <p><a name=\"WS73099CC142F487551FEA285E1221E4F9FF8-3BC1\"></a> <span class=\"code\">FbxLayerElement::EReferenceMode</span> - An enum that specifies how the IndexArray and DirectArray of a layer element are\n                        to be referenced. \n                     </p> \n                  </li>\n               </ul>\n            </div>\n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";