var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"FBX\">\n      <meta name=\"release\" content=\"2018\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2017-04-25\">\n      <meta name=\"topicid\" content=\"GUID-4F644045-380D-4B75-A2A3-D39DDE53BEDD\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>FBX Scenes</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-4F644045-380D-4B75-A2A3-D39DDE53BEDD\"></a><div class=\"head\">\n            <h1>FBX Scenes</h1>\n         </div>\n         <div class=\"section\"><a name=\"SECTION_80E2BF8B6C094C09AA34E21926AF5591\"></a> \n            <h2><a name=\"GUID-683B9D90-F1E9-4F37-8006-4ECB40427B8A\"></a>Scene Graph Organization Summary\n            </h2> \n            <p><a name=\"GUID-E5946729-CF54-4255-88CF-14455B4A30C8\"></a>The FBX SDK scene graph is abstracted by the <span class=\"code\">FbxScene</span> class. The scene is organized as a hierarchy of nodes (<span class=\"code\">FbxNode</span>). The root node of the scene is accessed via <span class=\"code\">FbxScene::GetRootNode()</span>. A scene element, for example, a mesh, a light, or a camera is defined by combining\n               a <span class=\"code\">FbxNode</span> with a subclass of <span class=\"code\">FbxNodeAttribute</span>. For more information, see <span class=\"char_link\"><a href=\"GUID-02CF7021-71FF-4F31-A072-291EF86CCA5A.htm\">FBX Nodes</a></span> and <span class=\"char_link\"><a href=\"GUID-FCD71AFF-F753-4D58-B96E-453ED84BF5C4.htm\">FBX Node Attributes</a></span>. \n            </p> \n            <p><a name=\"GUID-C77186C3-61EF-4879-A061-2CCE2B2604B3\"></a></p> <img src=\"../images/GUID-5FB404CF-20C8-4B44-B418-27234506276C-low.png\"> \n            <p><a name=\"GUID-2B260E63-0A6F-4DF5-AE98-B23EE87593FD\"></a> \n               <div><a name=\"GUID-52E8A91F-F8A7-408F-8281-292C811CF5BE\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span>During an export operation, the <em class=\"strong\">root node</em> of a scene is <em class=\"strong\">not exported</em> to the file. Only the children of the root node are exported to the file. As such,\n                     it is not recommended to associate anything to the root node that should be saved\n                     to a file. \n                  </div>\n               </div> \n            </p> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_01133A17816445E2B5E7525C03259E89\"></a> \n            <h2><a name=\"GUID-30BA14C0-D17D-4636-BE56-44EF5D2FB50E\"></a>Creating a Scene\n            </h2> \n            <p><a name=\"GUID-023AD1B7-9ADC-499F-9FE1-6E45258A3FD5\"></a>As illustrated in <span class=\"char_link\"><a href=\"GUID-B3BF2A41-2262-4706-ABB3-D3C861B3F03B.htm\">Managing Memory with the FBX SDK Manager</a></span>, a <span class=\"code\">FbxScene</span> is created by invoking the <span class=\"code\">FbxScene::Create()</span> function. A new <span class=\"code\">FbxScene</span> contains a root <span class=\"code\">FbxNode</span>, and a <span class=\"code\">FbxGlobalSettings</span> object with a default configuration. \n            </p> \n            <div class=\"codeBlock\"><pre>// Create the SDK manager.\nFbxManager* lSdkManager = FbxManager::Create();\n\n// Create the scene.\nFbxScene* lScene = FbxScene::Create(lSdkManager, \"Scene Name\");</pre></div> \n            <p><a name=\"GUID-492AD7CA-D886-44B7-A412-80187A6B3F95\"></a>Scene elements are created using a reference to the <span class=\"code\">FbxScene</span> in which they belong. In doing so, the scene will be exported with all the elements\n               which were created with it. In the following sample, a node is created with a reference\n               to the <span class=\"code\">FbxScene</span>, and attached to the root node of the scene. \n            </p> \n            <div class=\"codeBlock\"><pre>// Get the root node of the scene.\nFbxNode* lRootNode = lScene-&gt;GetRootNode();\n\n// Create a child node.\nFbxNode* lChild = FbxNode::Create(lScene, \"child\");\n\n// Add the child to the root node.\nlRootNode-&gt;AddChild(lChild);</pre></div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_E62CE2C147FB46EABD83805B1067422A\"></a> \n            <h2><a name=\"GUID-F9AD96D4-B026-4B38-BE98-DCDA0C510728\"></a>Global Scene Settings\n            </h2> \n            <p><a name=\"GUID-6F6C9A45-9010-4ED3-91AF-E3F4631FB5A4\"></a>The scene\'s axis system, system units, ambient lighting, and time settings are defined\n               in its <span class=\"code\">FbxGlobalSettings</span> object. This object is accessed via <span class=\"code\">FbxScene::GetGlobalSettings()</span>. \n            </p> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_C8DCF54B227E41CB9D687407A4CE87F5\"></a> \n            <h2><a name=\"GUID-73DB211B-CF87-4B93-9B1B-47E93937C3BB\"></a>Animation Evaluation\n            </h2> \n            <p><a name=\"GUID-726DAD57-DCD3-4903-BDC9-136FFE6CAE0D\"></a>The scene\'s <span class=\"code\">FbxAnimEvaluator</span> evaluates the animated geometric transformations of each node in the scene at a specific\n               time. It also evaluates any other animatable property of the scene, for example, the\n               color change of a specific material. The scene\'s <span class=\"code\">FbxAnimEvaluator</span> is accessed via <span class=\"code\">FbxScene::GetEvaluator()</span>. For more information, see <span class=\"char_link\"><a href=\"GUID-B3311B8D-5390-4C63-AB9F-662AC7D5C6CC.htm\">Animation</a></span>. \n            </p> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_685643685BE446D7BBD5D1F90B17BDD6\"></a> \n            <h2><a name=\"GUID-5E8F77B4-DCD7-4BF0-8788-030CE36A4BA2\"></a>Texture and Material Management\n            </h2> \n            <p><a name=\"GUID-2EEA1B94-A12F-4AA3-B927-6FD34955A468\"></a>The materials (<span class=\"code\">FbxSurfaceMaterial</span>) and textures (<span class=\"code\">FbxTexture</span>) created within the scene can be accessed and modified using member functions such\n               as <span class=\"code\">FbxScene::GetMaterial()</span> and <span class=\"code\">FbxScene::GetTexture()</span>. For more information on using materials and textures with meshes, see <span class=\"char_link\"><a href=\"GUID-8A69CD20-B8FB-4B23-925A-372E408390AA.htm\">Meshes, Materials and Textures</a></span>. \n            </p> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_7D89DB12C96345D8B8AB6D263704C372\"></a> \n            <h2><a name=\"GUID-C8B6034B-D717-459A-8E25-AE3DC27C63F8\"></a>Characters and Character Pose Management\n            </h2> \n            <p><a name=\"GUID-92F29AB2-6D6F-4045-8E53-DB501AF82ED3\"></a>Characters (<span class=\"code\">FbxCharacter</span>), and character poses (<span class=\"code\">FbxCharacterPose</span>) within the scene can be accessed using <span class=\"code\">FbxScene::GetCharacter()</span> and <span class=\"code\">FbxScene::GetCharacterPose()</span>. Consult the class documentation of <span class=\"code\">FbxCharacter</span> and <span class=\"code\">FbxCharacterPose</span> for more information on how to define characters in a scene. \n            </p> \n         </div><span class=\"noindex\">\n            <div class=\"topic-list\">\n               <h4 class=\"related\">Topics in this section</h4>\n               <ul class=\"jumplist\">\n                  <li class=\"topiclist-litem\"><a href=\"GUID-C08BAE05-B074-441C-9DBE-CE1811C0E6E2.htm\"> Merging Two Scenes </a><p>This topic presents how to merge two imported scenes. It also provides basic insight\n                        on manipulating a scene and the nodes within it, and will initiate the reader to the\n                        concept of connections and connection management. \n                        \n                     </p>\n                  </li>\n                  <li class=\"topiclist-litem\"><a href=\"GUID-CC93340E-C4A1-49EE-B048-E898F856CFBF.htm\">Scene Axis and Unit Conversion</a></li>\n               </ul>\n            </div></span><div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";