var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"FBX\">\n      <meta name=\"release\" content=\"2018\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2017-04-25\">\n      <meta name=\"topicid\" content=\"GUID-50489A8A-457C-4B54-80E1-5572A16F7F17\">\n      <meta name=\"indexterm\" content=\"io, i/o, settings, io settings, i/o settings, import, export\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>I/O Settings</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-50489A8A-457C-4B54-80E1-5572A16F7F17\"></a><div class=\"head\">\n            <h1>I/O Settings</h1>\n         </div>\n         <div class=\"section\"><a name=\"SECTION_3C0BFBDEF4DF41AEBBE2A95D55931A67\"></a> \n            <h2><a name=\"GUID-7FCE90CE-E01F-461F-ADFC-8729B86EE1FD\"></a>Creating an I/O Settings Object\n            </h2> \n            <p><a name=\"WS1A9193826455F5FF-150B16DA11960D83164-6A78\"></a> The <span class=\"code\">FbxIOSettings</span> class is responsible for specifying which elements of a scene are imported from a\n               file, or exported to a file. Such elements include cameras, lights, meshes, textures,\n               materials, animations, custom properties, etc. A <span class=\"code\">FbxIOSettings</span> object must be instantiated and configured before it is passed to a <span class=\"code\">FbxImporter</span> or <span class=\"code\">FbxExporter</span> object. Similarly to most objects in the FBX SDK, the <span class=\"code\">FbxIOSettings</span> object is created and managed using the <span class=\"code\">FbxManager</span> singleton (see <span class=\"char_link\"><a href=\"GUID-E802CA6C-4608-41ED-A35A-017CE6E9E8DD.htm\">FBX SDK Object Model</a></span>). \n            </p> \n            <div class=\"codeBlock\"><pre>#include &lt;fbxsdk.h&gt;\n#include &lt;fbxfilesdk/fbxio/fbxiosettings.h&gt;\n\n// ...\n\n// Create the FBX SDK manager\nFbxManager* lSdkManager = FbxManager::Create();\n\n// Create an IOSettings object. IOSROOT is defined in Fbxiosettingspath.h.\nFbxIOSettings * ios = FbxIOSettings::Create(lSdkManager, IOSROOT );\nlSdkManager-&gt;SetIOSettings(ios);</pre></div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_E150EC60E1414F738388AA39F5D2AFA3\"></a> \n            <h2><a name=\"GUID-D5B46D52-02CD-4DC0-AABE-84487D6B6173\"></a>Configuring the I/O Settings\n            </h2> \n            <p><a name=\"GUID-CBB5536D-D430-4930-A3AC-C97F17B7DDEA\"></a>The <span class=\"filePath\">Fbxiosettingspath.h</span> header file defines the <span class=\"code\">FbxIOSettings</span> configuration options. These configuration options are used as parameters in the\n               member functions of the <span class=\"code\">FbxIOSettings</span> class. Observe that the <span class=\"code\">IMP_</span> prefix pertains to import options, whereas the <span class=\"code\">EXP_</span> prefix pertains to export options. A newly instantiated <span class=\"code\">FbxIOSettings</span> object is configured with the default import and export options. \n            </p> \n            <div class=\"section\"><a name=\"SECTION_D016406ECAEB4088BAF26F0910F4055A\"></a> \n               <h3><a name=\"GUID-48B84FC2-C6E1-4BCD-A8CE-75D0FBA4E914\"></a>Import Settings\n               </h3> \n               <p><a name=\"GUID-1CAFC80C-0206-4B69-834F-ED3784967102\"></a>The following code snippet illustrates the configuration of the import settings with\n                  <span class=\"code\">FbxIOSettings::SetBoolProp()</span>. \n               </p> \n               <div class=\"codeBlock\"><pre>\n//&nbsp;Import&nbsp;options&nbsp;determine&nbsp;what&nbsp;kind&nbsp;of&nbsp;data&nbsp;is&nbsp;to&nbsp;be&nbsp;imported.\n// True is the default, but here we’ll set some to true explicitly, and others to false.\n(*(lSdkManager-&gt;GetIOSettings())).SetBoolProp(IMP_FBX_MATERIAL,&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;true);\n(*(lSdkManager-&gt;GetIOSettings())).SetBoolProp(IMP_FBX_TEXTURE,&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;true);\n(*(lSdkManager-&gt;GetIOSettings())).SetBoolProp(IMP_FBX_LINK,&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;false);\n(*(lSdkManager-&gt;GetIOSettings())).SetBoolProp(IMP_FBX_SHAPE,&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;false);\n(*(lSdkManager-&gt;GetIOSettings())).SetBoolProp(IMP_FBX_GOBO,&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;false);\n(*(lSdkManager-&gt;GetIOSettings())).SetBoolProp(IMP_FBX_ANIMATION,&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;true);\n(*(lSdkManager-&gt;GetIOSettings())).SetBoolProp(IMP_FBX_GLOBAL_SETTINGS,&nbsp;true);</pre></div> \n            </div> \n            <div class=\"section\"><a name=\"SECTION_FC4476C0022E461AA1EC43525F6F2026\"></a> \n               <h3><a name=\"GUID-1258BB5D-061E-4D9D-B270-AEB153A595FB\"></a>Export Settings\n               </h3> \n               <p><a name=\"WS1A9193826455F5FF2566FFD511FF6F8C7CA-6576\"></a>The following code snippet illustrates the configuration of the export settings with\n                  <span class=\"code\">FbxIOSettings::SetBoolProp()</span>. \n               </p> \n               <div class=\"codeBlock\"><pre>// Set the export states. By default, the export states are always set to \n// true except for the option eEXPORT_TEXTURE_AS_EMBEDDED. The code below \n// shows how to change these states.\nbool lEmbedMedia = true;\n(*(lSdkManager-&gt;GetIOSettings())).SetBoolProp(EXP_FBX_MATERIAL,        true);\n(*(lSdkManager-&gt;GetIOSettings())).SetBoolProp(EXP_FBX_TEXTURE,         true);\n(*(lSdkManager-&gt;GetIOSettings())).SetBoolProp(EXP_FBX_EMBEDDED,        lEmbedMedia);\n(*(lSdkManager-&gt;GetIOSettings())).SetBoolProp(EXP_FBX_SHAPE,           true);\n(*(lSdkManager-&gt;GetIOSettings())).SetBoolProp(EXP_FBX_GOBO,            true);\n(*(lSdkManager-&gt;GetIOSettings())).SetBoolProp(EXP_FBX_ANIMATION,       true);\n(*(lSdkManager-&gt;GetIOSettings())).SetBoolProp(EXP_FBX_GLOBAL_SETTINGS, true);</pre></div> \n            </div> \n            <p><a name=\"GUID-AB7D9150-F4DA-457D-B861-A3903B521374\"></a>The current import and export configuration options of a <span class=\"code\">FbxIOSettings</span> object may be analogously obtained using the <span class=\"code\">FbxIOSettings::GetBoolProp()</span>. \n            </p> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_CBFFD0BECB9C4E4FB47B1303A9CA19D5\"></a> \n            <h2><a name=\"GUID-1E1EA7EE-5708-458C-AEE0-58E9FDA7B88B\"></a>Embedding Media\n            </h2> \n            <p><a name=\"GUID-858BD392-FB18-4D11-8D6E-6B4731122232\"></a>The <span class=\"code\">FbxIOSettings</span> object\'s <span class=\"code\">EXP_FBX_EMBEDDED</span> property specifies how the FBX SDK should export any media (textures, sound, movies,\n               etc) related to the scene: \n               <div class=\"codeBlock\"><pre>bool lEmbedMedia = true;\n(*(lSdkManager-&gt;GetIOSettings())).SetBoolProp(EXP_FBX_EMBEDDED,        lEmbedMedia);</pre></div> <a name=\"WS1A9193826455F5FF-150B16DA11960D83164-6A66\"></a><p class=\"list-intro\"><a name=\"GUID-F7676A4C-4602-4C6A-9CFE-70670A2E067D\"></a>If the <span class=\"code\">EXP_FBX_EMBEDDED</span> property is set to:\n               </p>\n               <ul>\n                  <li><span class=\"code\">true</span>: the media is embedded within the exported file. This option is available only when\n                     exporting to binary FBX files. \n                  </li>\n                  <li><span class=\"code\">false</span>: the media is not embedded within the exported file. Instead, the FBX file contains\n                     relative references to the media files. \n                  </li>\n               </ul> \n            </p> \n            <p><a name=\"GUID-8C058049-6B9F-47BB-B3FC-A911975E90C6\"></a>When the FBX SDK imports a binary FBX file which contains embedded media (for example:\n               <span class=\"filePath\">myExportFile.fbx</span>), it will extract the embedded media files into the binary FBX file\'s <span class=\"filePath\">myExportFile.fbm/</span> subdirectory. If this subdirectory does not exist, it is automatically created. \n            </p> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_AB41D4C1E06E4D218235412E109E7294\"></a> \n            <h2><a name=\"GUID-AC5C563D-2038-4A51-8E6F-492E78EF48AE\"></a>Password Protection\n            </h2> \n            <p><a name=\"GUID-BF444E9C-3AD2-4725-A69B-AAFD15FBFCCB\"></a>The content of an FBX file can be password protected. This is achieved by setting\n               the appropriate string and boolean properties of the <span class=\"code\">FbxIOSettings</span> object before it is passed to a <span class=\"code\">FbxImporter</span> or <span class=\"code\">FbxExporter</span> object. \n            </p> \n            <div class=\"section\"><a name=\"SECTION_324F17A3A4134AF7A47D08806C37DE97\"></a> \n               <h3><a name=\"GUID-948A386C-E925-4D36-8D8C-64136E44F687\"></a>Password-Protected Import Settings\n               </h3> \n               <div class=\"codeBlock\"><pre>// The FbxString used to store the password.\nFbxString lString;\n\n// ... Assign the password to lString ...\n\n// Set the FbxIOSettings import password properties.\n(*(lSdkManager-&gt;GetIOSettings())).SetStringProp(IMP_FBX_PASSWORD, lString);\n(*(lSdkManager-&gt;GetIOSettings())).SetBoolProp(IMP_FBX_PASSWORD_ENABLE, true);</pre></div> \n            </div> \n            <div class=\"section\"><a name=\"SECTION_DDD4F0BB99B14A80820C3F65EC722204\"></a> \n               <h3><a name=\"GUID-026529AB-3C5C-4DD5-92C5-B9210A12A15E\"></a>Password-Protected Export Settings\n               </h3> \n               <div class=\"codeBlock\"><pre>// The FbxString used to store the password.\nFbxString lString;\n\n// ... Assign the password to lString ...\n\n// Set the FbxIOSettings import password properties.\n(*(lSdkManager-&gt;GetIOSettings())).SetStringProp(EXP_FBX_PASSWORD, lString);\n(*(lSdkManager-&gt;GetIOSettings())).SetBoolProp(EXP_FBX_PASSWORD_ENABLE, true);</pre></div> \n            </div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_8DAFA043C8074266A573104234114A27\"></a> \n            <h2><a name=\"GUID-CED65F6F-E0CE-4BE7-864F-0719610B22BC\"></a>Custom I/O Settings\n            </h2> \n            <p><a name=\"GUID-9AA82691-68C6-41DA-ABEF-0F32B0DCE60A\"></a>Custom I/O settings may be specified by adding <span class=\"code\">FbxProperty</span> objects to the <span class=\"code\">FbxIOSettings</span> object using methods such as <span class=\"code\">FbxIOSettings::AddProperty()</span> and <span class=\"code\">FbxIOSettings::AddPropertyGroup()</span>. The underlying <span class=\"code\">FbxProperty</span> tree structure of the <span class=\"code\">FbxIOSettings</span> class is a relevant concept in understanding how to define your own custom I/O settings.\n               Consult the <span class=\"code\">FbxIOSettings</span> class documentation for more details. \n            </p> \n         </div>\n         <div><a name=\"GUID-5512E198-8610-4346-8ED4-D9EAD9F4197F\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span>The FBX SDK does not directly support exporting and importing to/from a memory stream.\n               \n            </div>\n         </div>\n         <div class=\"see-also\">\n            <h4>See Also</h4>\n            <div class=\"related-links\">\n               <ul>\n                  <li><span class=\"char_link\"><a href=\"GUID-ABFC58F1-C10D-42AE-A204-D79EB1D84B24.htm\">Importing a Scene </a></span></li>\n                  <li><span class=\"char_link\"><a href=\"GUID-4E8F95CA-CD9B-4E24-B8AD-7C399EA78137.htm\">Exporting a Scene </a></span></li>\n                  <li><span class=\"char_link\"><a href=\"GUID-75CD0DC4-05C8-4497-AC6E-EA11406EAE26.htm\">Customizing File Formats with FBX SDK I/O Plug-ins </a></span></li>\n               </ul>\n            </div>\n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";