var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"FBX\">\n      <meta name=\"release\" content=\"2018\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2017-04-25\">\n      <meta name=\"topicid\" content=\"GUID-5A05DFB2-4985-4FE5-B0EF-C2855F925AAF\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Cameras</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-5A05DFB2-4985-4FE5-B0EF-C2855F925AAF\"></a><div class=\"head\">\n            <h1>Cameras</h1>\n         </div>\n         <div class=\"section\"><a name=\"SECTION_0DE7970C57424F1FABED44A14B701EDC\"></a> \n            <h2><a name=\"GUID-CF8FB32E-7996-4284-A427-1455DFFEA35F\"></a>Creating a Camera\n            </h2> \n            <p><a name=\"GUID-A9C7A3CC-9525-4D9B-8BA1-1FB55C25834F\"></a>Cameras in the FBX SDK are abstracted by the <span class=\"code\">FbxCamera</span> class. Stereocameras for 3D imaging are abstracted by the <span class=\"code\">FbxCameraStereo</span> class. In this topic, we will only look at the basic ways to create and manipulate\n               a <span class=\"code\">FbxCamera</span>. \n            </p> \n            <p><a name=\"GUID-C471A1CE-0BC0-42EA-AC47-41E8D39627B7\"></a>By default, a <span class=\"code\">FbxCamera</span> points in the direction of the <em class=\"strong\">node\'s positive X axis</em>. \n            </p> \n            <div class=\"codeBlock\"><pre>// Create a node for our camera in the scene.\nFbxNode* lCameraNode = FbxNode::Create(pScene, \"cameraNode\");\n\n// Create a light.\nFbxCamera* lCamera = FbxCamera::Create(pScene, \"camera\");\n\n// Set the node attribute of the camera node.\nlCameraNode-&gt;SetNodeAttribute(lCamera);\n\n// Add the camera node to the root node in the scene.\nFbxNode* lRootNode = pScene-&gt;GetRootNode();\nlRootNode-&gt;AddChild(lCameraNode);</pre></div> \n            <p><a name=\"GUID-062216B9-C2C0-420D-842B-17A7A32C7B76\"></a>Once a camera has been created, it can be set as the scene\'s default camera. A scene\n               must have its default camera set explicitly, even if there is only one camera in the\n               scene. \n            </p> \n            <div class=\"codeBlock\"><pre>// Set the scene\'s default camera.\npScene-&gt;GetGlobalSettings().SetDefaultCamera((char *) lCamera-&gt;GetName());</pre></div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_319B419EE9C64EEDBFE585A72C3003CE\"></a> \n            <h2><a name=\"GUID-72D7D34A-B4DD-4DF2-A5A2-6EBBEAD18567\"></a>Pointing a Camera\n            </h2> \n            <p><a name=\"GUID-F980BA12-6B75-4F9D-A7F3-863A2A72501B\"></a> A camera can be forced to consistently point towards a specific target in the scene.\n               To do this, the camera\'s node must have its target set using <span class=\"code\">FbxNode::SetTarget()</span>. The <span class=\"code\">FbxMarker</span> node attribute is used in the target node. \n            </p> \n            <div class=\"codeBlock\"><pre>// Create a node to contain the marker. This will be our target node.\nFbxNode* lTargetNode = FbxNode::Create(pScene, \"targetNode\");\n\n// Create a marker node attribute.\nFbxMarker* lMarker = FbxMarker::Create(pScene, \"cameraMarker\");\n\n// Set the marker as the target node\'s attribute.\nlTargetNode-&gt;SetNodeAttribute(lMarker);\n\n// Set our camera node\'s target.\nlCameraNode-&gt;SetTarget(lTargetNode);</pre></div> \n            <p><a name=\"GUID-DDFDC854-A29C-4C77-9E04-D0282EFBA73A\"></a> \n               <div><a name=\"GUID-947211B2-4ADE-41C4-BB7E-54DCA797DEAF\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span>Consult the \"<em class=\"strong\">Node Target Management</em>\" section in the <span class=\"code\">FbxNode</span> class documentation for more information. \n                  </div>\n               </div> \n            </p> \n            <p><a name=\"GUID-96F42CD2-AC0C-480C-B471-15E020BF2E41\"></a>By default, the <span class=\"code\">FbxCamera::FocusSource</span> property is set to <span class=\"code\">FbxCamera::eFocusSrcCameraInterest</span> to maintain focus on the camera\'s target. The focus source can also be set to a specific\n               distance from the camera. \n            </p> \n            <div class=\"codeBlock\"><pre>// Set the camera\'s focus source to a specific distance.\nlCamera-&gt;FocusSource.Set(FbxCamera::eFocusSpecificDistance);\n\n// Set the distance to 100.0 units from the camera. \n// The default value for this distance is 200.0 units.\nlCamera-&gt;FocusDistance.Set(100.0);</pre></div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_A95B1B63DD6243EBA0E9A070A9826743\"></a> \n            <h2><a name=\"GUID-CCE70446-5403-4B76-A8F6-A00DAC8D3B65\"></a>Camera Properties\n            </h2> \n            <p><a name=\"GUID-F7F7C765-8707-4E3C-89B9-4AFE97DEBBE3\"></a>Consult the <span class=\"code\">FbxCamera</span> class documentation for more information on configuring cameras in a scene. \n            </p> \n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";