var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"FBX\">\n      <meta name=\"release\" content=\"2018\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2017-04-25\">\n      <meta name=\"topicid\" content=\"GUID-5EDC0280-E000-4B0B-88DF-5D215A589D5E\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Meshes</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-5EDC0280-E000-4B0B-88DF-5D215A589D5E\"></a><div class=\"head\">\n            <h1>Meshes</h1>\n         </div>\n         <div class=\"section\"><a name=\"SECTION_F4FBAE47DFEF4985ABD92DB99D9E59C1\"></a> \n            <h2><a name=\"GUID-1BAE1C3F-9B73-4CFA-821B-B2C3BFB8EAE9\"></a>Creating a Mesh\n            </h2> \n            <p><a name=\"GUID-1C25CB77-BA5E-4B67-81C2-1E87BF0B5851\"></a>Meshes in the FBX SDK are abstracted by the <span class=\"code\">FbxMesh</span> class. A mesh is defined by a set of per-face vertices, or \"control points\", and\n               by a group of layers to define the mesh\'s normals, textures, and materials. The following\n               code sample illustrates how to instantiate a <span class=\"code\">FbxMesh</span>. \n            </p> \n            <div class=\"codeBlock\"><pre>// Create a node for our mesh in the scene.\nFbxNode* lMeshNode = FbxNode::Create(pScene, \"meshNode\");\n\n// Create a mesh.\nFbxMesh* lMesh = FbxMesh::Create(pScene, \"mesh\");\n\n// Set the node attribute of the mesh node.\nlMeshNode-&gt;SetNodeAttribute(lMesh);\n\n// Add the mesh node to the root node in the scene.\nFbxNode *lRootNode = pScene-&gt;GetRootNode();\nlRootNode-&gt;AddChild(lMeshNode);</pre></div> \n            <p><a name=\"GUID-DE8A234A-7BA0-42C3-BA0E-34DDEA0C26BE\"></a></p> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_04C6D80542E14538B1087383AE77A187\"></a> \n            <h2><a name=\"GUID-E546A325-19C8-46C4-8347-A3998133DBB9\"></a>Defining Control Points\n            </h2> \n            <p><a name=\"GUID-1EEAF31B-95A9-4FF0-A03D-BCCEB11406B9\"></a>The per-face vertices of a <span class=\"code\">FbxMesh</span> are known as <em class=\"mild\">control points</em>. Since objects in the FBX SDK are created by default in the <em class=\"strong\">right handed, Y-Up axis system</em>, the vertices of a mesh should be defined accordingly. An instance of <span class=\"code\">FbxMesh</span> contains an array of control points which can be initialized to a specific size.\n               For example, a cube\'s mesh requires 24 control points: there are 4 control points\n               per face, and a cube has 6 faces. The following code sample initializes the array\n               of control points for a cube. \n            </p> \n            <div class=\"codeBlock\"><pre>// Define the eight corners of the cube.\n// The cube spans from\n//    -50 to  50 along the X axis\n//      0 to 100 along the Y axis\n//    -50 to  50 along the Z axis\nFbxVector4 vertex0(-50, 0, 50);\nFbxVector4 vertex1(50, 0, 50);\nFbxVector4 vertex2(50, 100, 50);\nFbxVector4 vertex3(-50, 100, 50);\nFbxVector4 vertex4(-50, 0, -50);\nFbxVector4 vertex5(50, 0, -50);\nFbxVector4 vertex6(50, 100, -50);\nFbxVector4 vertex7(-50, 100, -50);\n\n// Initialize the control point array of the mesh.\nlMesh-&gt;InitControlPoints(24);\nFbxVector4* lControlPoints = lMesh-&gt;GetControlPoints();\n\n// Define each face of the cube.\n// Face 1\nlControlPoints[0] = vertex0;\nlControlPoints[1] = vertex1;\nlControlPoints[2] = vertex2;\nlControlPoints[3] = vertex3;\n// Face 2\nlControlPoints[4] = vertex1;\nlControlPoints[5] = vertex5;\nlControlPoints[6] = vertex6;\nlControlPoints[7] = vertex2;\n// Face 3\nlControlPoints[8] = vertex5;\nlControlPoints[9] = vertex4;\nlControlPoints[10] = vertex7;\nlControlPoints[11] = vertex6;\n// Face 4\nlControlPoints[12] = vertex4;\nlControlPoints[13] = vertex0;\nlControlPoints[14] = vertex3;\nlControlPoints[15] = vertex7;\n// Face 5\nlControlPoints[16] = vertex3;\nlControlPoints[17] = vertex2;\nlControlPoints[18] = vertex6;\nlControlPoints[19] = vertex7;\n// Face 6\nlControlPoints[20] = vertex1;\nlControlPoints[21] = vertex0;\nlControlPoints[22] = vertex4;\nlControlPoints[23] = vertex5;</pre></div> \n            <p><a name=\"GUID-9785A762-07C7-4C90-BC79-644991A59F3E\"></a>For more information on control point management, consult the <span class=\"code\">FbxGeometry</span> class documentation. <span class=\"code\">FbxGeometry</span> is the parent class of <span class=\"code\">FbxMesh</span>, <span class=\"code\">FbxLine</span>, <span class=\"code\">FbxNurb</span> and other classes used to define scene geometry. \n            </p> \n            <div><a name=\"GUID-B1621FEF-C94C-459F-8B88-2EC1D88774B8\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span>Note that a scene axis conversion does not affect the vertex values of a mesh. For\n                  more information, see <span class=\"char_link\"><a href=\"GUID-CC93340E-C4A1-49EE-B048-E898F856CFBF.htm\">Scene Axis and Unit Conversion</a></span>. \n               </div>\n            </div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_9A92386E8710434FB8D062862ED7F057\"></a> \n            <h2><a name=\"GUID-9B1EF480-17FE-4F7B-BB69-2F2A880EB49E\"></a>Assigning Normals\n            </h2> \n            <p><a name=\"GUID-60BD8A9E-BAE1-45F6-8C55-CC7A60F38EA3\"></a>The normal vectors of a mesh are defined in an instance of <span class=\"code\">FbxLayerElementNormal</span>. Layer elements such as normals can be mapped to the surface of a mesh in a variety\n               of ways, such as by control point (<span class=\"code\">FbxLayerElement::eByControlPoint</span>), by polygon vertex (<span class=\"code\">FbxLayerElement::eByPolygonVertex</span>), by polygon (<span class=\"code\">FbxLayerElement::eByPolygon</span>), by edge (<span class=\"code\">FbxLayerElement::eByEdge</span>), or one mapping coordinate for the whole surface (<span class=\"code\">FbxLayerElement::eAllSame</span>). For more information, see <span class=\"code\">FbxLayerElement::EMappingMode</span>. \n            </p> \n            <p><a name=\"GUID-027F03DA-1853-4618-AB68-E3BB12BD2FA1\"></a>Given an array of <em class=\"strong\">normal vectors</em> and an array of <em class=\"strong\">control points</em>, we can specify how the array of normal vectors is referenced by the array of control\n               points. This is defined by the <span class=\"code\">FbxLayerElement::EReferenceMode</span> assigned in <span class=\"code\">FbxLayerElement::SetReferenceMode()</span>. The <span class=\"code\">EReferenceMode::eDirect</span> mode maps the n\'th element in the normal vector array to the n\'th element in the\n               control point array. \n            </p> <a name=\"WS1A9193826455F5FF-150B16DA11960D83164-6C23\"></a><div class=\"table-ruled\">\n               <table cellpadding=\"0\" cellspacing=\"0\" class=\"ruled\">\n                  <colgroup>\n                     <col width=\"28.729066597429576%\">\n                     <col width=\"71.27093340257042%\">\n                  </colgroup>\n                  <tr class=\"ruled-heading\">\n                     <th class=\"table-heading\">Reference Mode (<span class=\"code\">FbxLayerElement::EReferenceMode</span>) \n                     </th>\n                     <th class=\"table-heading\">Description </th>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <span class=\"code\">FbxLayerElement::eDirect</span> \n                     </td>\n                     <td class=\"table-body\"> This indicates that the mapping information for the n\'th element is found in the\n                        n\'th place of <span class=\"code\">FbxLayerElementTemplate::mDirectArray</span>. \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-even-row\">\n                     <td class=\"table-body\"> <span class=\"code\">FbxLayerElement::eIndex</span> \n                     </td>\n                     <td class=\"table-body\">This symbol is kept for backward compatibility with FBX v5.0 files. In FBX v6.0 and\n                        higher, this symbol is replaced with <span class=\"code\">FbxLayerElement::eIndexToDirect</span>. \n                     </td>\n                  </tr>\n                  <tr class=\"ruled-odd-row\">\n                     <td class=\"table-body\"> <span class=\"code\">FbxLayerElement::eIndexToDirect</span> \n                     </td>\n                     <td class=\"table-body\">This indicates that each element of <span class=\"code\">FbxLayerElementTemplate::mIndexArray</span> contains an index referring to an element in <span class=\"code\">FbxLayerElementTemplate::mDirectArray</span>. \n                     </td>\n                  </tr>\n               </table>\n            </div> \n            <p><a name=\"GUID-C5F30BC1-86EB-481F-9835-F6799E9AF9EB\"></a>In the following code sample, we will examine how to apply a per-control point mapping\n               (<span class=\"code\">FbxLayerElement::eByControlPoint</span>) of normals onto a mesh using a <span class=\"code\">EReferenceMode::eDirect</span> reference mode. Observe that the order in which <span class=\"code\">FbxLayerElementArrayTemplate::Add()</span> is invoked corresponds to the order of the control points which have been defined\n               above. \n            </p> \n            <div class=\"codeBlock\"><pre>// Define normal vectors along each axis.\nFbxVector4 lNormalXPos( 1,  0,  0);\nFbxVector4 lNormalXNeg(-1,  0,  0);\nFbxVector4 lNormalYPos( 0,  1,  0);\nFbxVector4 lNormalYNeg( 0, -1,  0);\nFbxVector4 lNormalZPos( 0,  0,  1);\nFbxVector4 lNormalZNeg( 0,  0, -1);\n\n// Create layer 0 for the mesh if it does not already exist.\n// This is where we will define our normals.\nFbxLayer lLayer = lMesh-&gt;GetLayer(0);\nif(lLayer == NULL) {\n    lMesh-&gt;CreateLayer();\n    lLayer = lMesh-&gt;GetLayer(0);\n}\n\n// Create a normal layer.\nFbxLayerElementNormal* lLayerElementNormal= FbxLayerElementNormal::Create(lMesh, \"\");\n\n// Set its mapping mode to map each normal vector to each control point.\nlLayerElementNormal-&gt;SetMappingMode(FbxLayerElement::eByControlPoint);\n \n// Set the reference mode of so that the n\'th element of the normal array maps to the n\'th\n// element of the control point array.\nlLayerElementNormal-&gt;SetReferenceMode(FbxLayerElement::eDirect);\n\n// Assign the normal vectors in the same order the control points were defined for the mesh.\n// Face 1\nlLayerElementNormal-&gt;GetDirectArray().Add(lNormalZPos);\nlLayerElementNormal-&gt;GetDirectArray().Add(lNormalZPos);\nlLayerElementNormal-&gt;GetDirectArray().Add(lNormalZPos);\nlLayerElementNormal-&gt;GetDirectArray().Add(lNormalZPos);\n// Face 2\nlLayerElementNormal-&gt;GetDirectArray().Add(lNormalXPos);\nlLayerElementNormal-&gt;GetDirectArray().Add(lNormalXPos);\nlLayerElementNormal-&gt;GetDirectArray().Add(lNormalXPos);\nlLayerElementNormal-&gt;GetDirectArray().Add(lNormalXPos);\n// Face 3\nlLayerElementNormal-&gt;GetDirectArray().Add(lNormalZNeg);\nlLayerElementNormal-&gt;GetDirectArray().Add(lNormalZNeg);\nlLayerElementNormal-&gt;GetDirectArray().Add(lNormalZNeg);\nlLayerElementNormal-&gt;GetDirectArray().Add(lNormalZNeg);\n// Face 4\nlLayerElementNormal-&gt;GetDirectArray().Add(lNormalXNeg);\nlLayerElementNormal-&gt;GetDirectArray().Add(lNormalXNeg);\nlLayerElementNormal-&gt;GetDirectArray().Add(lNormalXNeg);\nlLayerElementNormal-&gt;GetDirectArray().Add(lNormalXNeg);\n// Face 5\nlLayerElementNormal-&gt;GetDirectArray().Add(lNormalYPos);\nlLayerElementNormal-&gt;GetDirectArray().Add(lNormalYPos);\nlLayerElementNormal-&gt;GetDirectArray().Add(lNormalYPos);\nlLayerElementNormal-&gt;GetDirectArray().Add(lNormalYPos);\n// Face 6\nlLayerElementNormal-&gt;GetDirectArray().Add(lNormalYNeg);\nlLayerElementNormal-&gt;GetDirectArray().Add(lNormalYNeg);\nlLayerElementNormal-&gt;GetDirectArray().Add(lNormalYNeg);\nlLayerElementNormal-&gt;GetDirectArray().Add(lNormalYNeg);\n\n// Finally, we set layer 0 of the mesh to the normal layer element.\nlLayer-&gt;SetNormals(lLayerElementNormal);</pre></div> \n         </div><span class=\"noindex\">\n            <div class=\"topic-list\">\n               <h4 class=\"related\">Topics in this section</h4>\n               <ul class=\"jumplist\">\n                  <li class=\"topiclist-litem\"><a href=\"GUID-0D483705-23D9-476D-A567-09609396B190.htm\">Instancing - Sharing a Mesh</a></li>\n               </ul>\n            </div></span><div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";