var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"FBX\">\n      <meta name=\"release\" content=\"2018\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2017-04-25\">\n      <meta name=\"topicid\" content=\"GUID-6006F800-34F3-47B2-A208-E36DED53BEB3\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Exporting with the FBX Extensions SDK</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-6006F800-34F3-47B2-A208-E36DED53BEB3\"></a><div class=\"head\">\n            <h1>Exporting with the FBX Extensions SDK</h1>\n         </div>\n         <div class=\"section\"><a name=\"SECTION_2E674C6CF7474C4CB0D5B90C2839DCB6\"></a> \n            <h2><a name=\"GUID-1E19D3D1-805D-46A3-A784-753417430A71\"></a>Host Application Flow\n            </h2> \n            <p><a name=\"GUID-9A7F552B-7CF9-43B1-9D10-71FC6C4BB024\"></a>The following pseudocode shows the order of execution of an FBX Extension while a\n               scene is being exported by the host application. \n            </p> \n            <div class=\"codeBlock\"><pre>\nBefore translating any scene objects to FBX:\n&nbsp;&nbsp;&nbsp;&nbsp;ExportBegin();\n&nbsp;\nFor each object in the scene:\n&nbsp;&nbsp;&nbsp;&nbsp;If ( ExportHandled() returns TRUE ):\n&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;&nbsp;The host application will let your extension translate the object.\n&nbsp;&nbsp;&nbsp;&nbsp;Else:\n        The host application will translate the object, if it knows how to.\n&nbsp;\nFor each object translated by the host application:\n&nbsp;&nbsp;&nbsp;&nbsp;ExportTranslated(), passing the object and its translation;\n&nbsp;\nThe translation of the scene is now complete:\n&nbsp;&nbsp;&nbsp;&nbsp;ExportEnd()\n</pre></div> \n         </div>\n         <div class=\"section\"><a name=\"WS1A9193826455F5FF2566FFD511FDE75E6567490\"></a> \n            <h2><a name=\"GUID-41C8D237-0468-4FEB-B35B-F6E7837CA2EA\"></a>ExportBegin()\n            </h2> \n            <div class=\"codeBlock\"><pre>\nEXPORT_DLL void MayaExt_ExportBegin( FbxScene* pFbxScene );\nEXPORT_DLL void MaxExt_ExportBegin( FbxScene* pFbxScene, INode* pMaxRootNode );\nEXPORT_DLL void MBExt_ExportBegin( FbxScene* pFbxScene );\n</pre></div> \n            <p><a name=\"WS1A9193826455F5FF2566FFD511FDE75E6567491\"></a>This function is called before the host application translates any data from the scene\n               into the corresponding FBX scene. The host application passes a pointer to the empty\n               FBX scene that it has created. Your FBX Extension can access the 3ds Max/Maya/MotionBuilder\n               scene by calling functions in the 3ds Max/Maya/MotionBuilder SDKs. \n            </p> \n            <p><a name=\"WS1A9193826455F5FF2566FFD511FDE75E6567492\"></a>Your FBX Extension can create your custom data now, or wait at the end of the export\n               depending if that custom data replaces objects in the FBX scene hierarchy. \n            </p> \n         </div>\n         <div class=\"section\"><a name=\"WS1A9193826455F5FF2566FFD511FDE75E6567493\"></a> \n            <h2><a name=\"GUID-706942C6-72BB-4186-BC4A-B78F66EB1B02\"></a>ExportHandled()\n            </h2> \n            <div class=\"codeBlock\"><pre>\nEXPORT_DLL bool MayaExt_ExportHandled( MObject&amp; pMayaObject );\nEXPORT_DLL bool MaxExt_ExportHandled( INode* pMaxObject );\nEXPORT_DLL bool MBExt_ExportHandled( HFBComponent pFBComponent );\n</pre></div> \n            <p><a name=\"WS1A9193826455F5FF2566FFD511FDE75E6567494\"></a>Called for each object in the scene. Called <em class=\"mild\">before</em> that object is translated into an FBX object. May be called more than once for the\n               same object. \n            </p> <a name=\"WS1A9193826455F5FF2566FFD511FF6F8C7CA-7FE7\"></a><p class=\"list-intro\"><a name=\"WS1A9193826455F5FF2566FFD511FF6F8C7CA-7FE6\"></a>Your extension must return:\n            </p>\n            <ul>\n               <li>true, if your extension is responsible for creating the corresponding FBX object for\n                  this 3ds Max/Maya/MotionBuilder object; the host application will not translate the\n                  object. \n               </li>\n               <li>false: the host application will translate the object (providing that it knows how\n                  to translate objects of that type). \n               </li>\n            </ul> \n         </div>\n         <div class=\"section\"><a name=\"WS1A9193826455F5FF2566FFD511FDE75E6567497\"></a> \n            <h2><a name=\"GUID-81ABA518-0149-4C7E-8FFC-EBC092FD27AC\"></a>ExportTranslated()\n            </h2> \n            <div class=\"codeBlock\"><pre>\nEXPORT_DLL void MayaExt_ExportTranslated( FbxObject* pFbxObject, MObject&amp; pMayaObject );\nEXPORT_DLL void MaxExt_ExportTranslated( FbxObject* pFbxObject, INode* pMaxObject );\nEXPORT_DLL void MBExt_ExportTranslated( FbxObject* pFbxObject, HFBComponent pFBComponent );\n</pre></div> \n            <p><a name=\"WS1A9193826455F5FF2566FFD511FDE75E6567498\"></a>After all objects in the scene have been translated into FBX objects, the host application\n               calls this function once for each object that has been translated. \n            </p> \n            <p><a name=\"WS1A9193826455F5FF2566FFD511FDE75E6567499\"></a>The purpose of this function is to tell the extension: for this 3ds Max/Maya/MotionBuilder\n               object, this is the corresponding FBX object. \n            </p> \n         </div>\n         <div class=\"section\"><a name=\"WS1A9193826455F5FF2566FFD511FDE75E656749A\"></a> \n            <h2><a name=\"GUID-1D9D67D6-3721-4474-B3AB-0CC8B395B75A\"></a>ExportEnd()\n            </h2> \n            <div class=\"codeBlock\"><pre>\nEXPORT_DLL void MayaExt_ExportEnd( FbxScene* pFbxScene );\nEXPORT_DLL void MaxExt_ExportEnd( FbxScene* pFbxScene, INode* pMaxRootNode );\nEXPORT_DLL void MBExt_ExportEnd( FbxScene* pFbxScene );\n</pre></div> \n            <p><a name=\"WS1A9193826455F5FF2566FFD511FDE75E656749B\"></a>The host application calls this function after it has completely translated the scene\n               into FBX (and before the file I/O begins). This is the last chance for your extension\n               to modify the FBX scene. \n            </p> \n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";