var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"FBX\">\n      <meta name=\"release\" content=\"2018\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2017-04-25\">\n      <meta name=\"topicid\" content=\"GUID-75CD0DC4-05C8-4497-AC6E-EA11406EAE26\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Customizing File Formats with FBX SDK I/O Plug-ins</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-75CD0DC4-05C8-4497-AC6E-EA11406EAE26\"></a><div class=\"head\">\n            <h1>Customizing File Formats with FBX SDK I/O Plug-ins</h1>\n         </div>\n         <div class=\"section\"><a name=\"SECTION_0AA21F13C461496D90BCE15FA5C54E4B\"></a> \n            <h2><a name=\"GUID-7EB8F3AA-A20F-46B6-8021-4A9189151793\"></a>Loading an FBX SDK I/O Plug-in\n            </h2> \n            <p><a name=\"GUID-DD4DFD92-9FEE-40F7-B7A5-E88CA20AF928\"></a>The FBX SDK uses a plug-in architecture to implement its file reading and writing\n               capabilities. FBX SDK I/O Plug-ins consist of dynamically loaded library (DLL) files\n               which can be loaded using the <span class=\"code\">FbxManager::LoadPluginsDirectory()</span> method. The following code can be found in the <span class=\"filePath\">Common/Common.cxx</span> sample. \n            </p> \n            <div class=\"codeBlock\"><pre>void InitializeSdkObjects(FbxManager*&amp; pSdkManager, FbxScene*&amp; pScene)\n{\n    // The first thing to do is to create the FBX SDK manager which is the \n    // object allocator for almost all the classes in the SDK.\n    pSdkManager = FbxManager::Create();\n\n    if (!pSdkManager)\n    {\n        printf(\"Unable to create the FBX SDK manager\\n\");\n        exit(0);\n    }\n\n        // create an IOSettings object\n        FbxIOSettings * ios = FbxIOSettings::Create(pSdkManager, IOSROOT );\n        pSdkManager-&gt;SetIOSettings(ios);\n\n        // Load plugins from the executable directory\n        FbxString lPath = FbxGetApplicationDirectory();\n#if defined(KARCH_ENV_WIN)\n        FbxString lExtension = \"dll\";\n#elif defined(KARCH_ENV_MACOSX)\n        FbxString lExtension = \"dylib\";\n#elif defined(KARCH_ENV_LINUX)\n        FbxString lExtension = \"so\";\n#endif\n        pSdkManager-&gt;LoadPluginsDirectory(lPath.Buffer(), lExtension.Buffer());\n\n    // Create the entity that will hold the scene.\n    pScene = FbxScene::Create(pSdkManager,\"\");\n}</pre></div> \n            <p><a name=\"GUID-AFE74610-4874-41C5-9F8D-519E1F62E16F\"></a>An FBX SDK I/O Plug-in inherits from the <span class=\"code\">FbxPlugin</span> class. Consult the <span class=\"code\">FbxPlugin</span> class documentation for a template FBX SDK I/O Plug-in. \n            </p> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_9CC1FF5953194740B3444784D738AC94\"></a> \n            <h2><a name=\"GUID-7ABA1A06-FBB9-40CF-8701-0DB9BFFB51B0\"></a>Custom File Writer and Reader Plug-in\n            </h2> \n            <p><a name=\"GUID-C48A7F78-7A33-4C0C-AB25-BEB000482929\"></a>A reference to the <span class=\"code\">FbxManager</span> singleton object can be accessed from within your FBX SDK I/O Plug-in with <span class=\"code\">FbxPlugin::GetData()</span>. \n            </p> \n            <p><a name=\"GUID-04D8FC28-E0C7-48BB-84E4-53DF7995E93A\"></a>The <span class=\"code\">FbxManager</span>\'s I/O plug-in registry can be accessed with <span class=\"code\">FbxManager::GetIOPluginRegistry()</span>. The <span class=\"code\">FbxIOPluginRegistry</span> indexes the <span class=\"code\">FbxReader</span> and <span class=\"code\">FbxWriter</span> objects used to read/write to specific file formats. You can register your own custom\n               subclasses of <span class=\"code\">FbxReader</span> and <span class=\"code\">FbxWriter</span> by calling <span class=\"code\">FbxIOPluginRegistry::RegisterReader()</span> and <span class=\"code\">FbxIOPluginRegistry::RegisterWriter()</span>. \n            </p> \n            <p><a name=\"GUID-34267BA5-C033-44F0-BC96-EFE3E623C3FE\"></a>The following sample (present in the program below) illustrates how your own <span class=\"code\">FbxReader</span> and <span class=\"code\">FbxWriter</span> can be registered into the <span class=\"code\">FbxManager</span>\'s <span class=\"code\">FbxIOPluginRegistry</span>: \n               <div class=\"codeBlock\"><pre>class MyOwnWriterReaderPlugin : public FbxPlugin\n{\n    FBXPLUGIN_DECLARE(MyOwnWriterReaderPlugin);\n\nprotected:\n    explicit MyOwnWriterReaderPlugin(const FbxPluginDefinition&amp; pDefinition, kLibHandle pLibHandle) : FbxPlugin(pDefinition, pLibHandle)\n    {\n    }\n\n    // Implement Fbxmodules::FbxPlugin\n    virtual bool SpecificInitialize()\n    {\n        int FirstPluginID, RegistredCount;\n        GetData().mSDKManager-&gt;GetIOPluginRegistry()-&gt;RegisterReader(CreateMyOwnReader, GetMyOwnReaderInfo, FirstPluginID, RegistredCount, FillOwnReaderIOSettings);\n        GetData().mSDKManager-&gt;GetIOPluginRegistry()-&gt;RegisterWriter(CreateMyOwnWriter, GetMyOwnWriterInfo, FirstPluginID, RegistredCount, FillOwnWriterIOSettings);\n        return true;\n    }\n\n    virtual bool SpecificTerminate()\n    {\n        return true;\n    }\n};</pre></div> \n            </p> \n            <p><a name=\"GUID-03725BDA-E534-4051-9C01-52BE9EF62AEC\"></a> The <span class=\"char_link\"><a href=\"GUID-4E8F95CA-CD9B-4E24-B8AD-7C399EA78137.htm#SECTION_B81B4BF3CCD54E1B87C38835A6C9F5F0\">Exporting With Embedded Media</a></span> subsection shows how the <span class=\"code\">FbxIOPluginRegistry</span> can be used to specify the desired file format of a <span class=\"code\">FbxExporter</span>. \n            </p> \n            <div><a name=\"GUID-099E837B-70E7-45F8-9FB3-01D79B9590C1\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span>The <span class=\"filePath\">MyOwnWriterReader</span> sample provides insight on how to customize your own <span class=\"code\">FbxReader</span> and <span class=\"code\">FbxWriter</span> classes. \n               </div>\n            </div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_0F975C18C09D431F8D4A1F11D2845D85\"></a> \n            <h2><a name=\"GUID-66859250-3ADA-402F-AC5B-1BA1153B55E8\"></a>Sample FBX SDK I/O Plug-in\n            </h2> \n            <p><a name=\"GUID-9789336A-2E10-432B-8A16-82F3F7316ABE\"></a>The code below shows how to implement a sample FBX SDK I/O Plug-in. It uses the <span class=\"filePath\">MyOwnWriterReader</span> example to implement custom versions of <span class=\"code\">FbxReader</span> and <span class=\"code\">FbxWriter</span>. \n            </p> \n            <div class=\"codeBlock\"><pre>/**************************************************************************************\n\n Copyright (C) 2001 - 2010 Autodesk, Inc. and/or its licensors.\n All Rights Reserved.\n\n The coded instructions, statements, computer programs, and/or related material \n (collectively the \"Data\") in these files contain unpublished information \n proprietary to Autodesk, Inc. and/or its licensors, which is protected by \n Canada and United States of America federal copyright law and by international \n treaties. \n \n The Data may not be disclosed or distributed to third parties, in whole or in\n part, without the prior written consent of Autodesk, Inc. (\"Autodesk\").\n\n THE DATA IS PROVIDED \"AS IS\" AND WITHOUT WARRANTY.\n ALL WARRANTIES ARE EXPRESSLY EXCLUDED AND DISCLAIMED. AUTODESK MAKES NO\n WARRANTY OF ANY KIND WITH RESPECT TO THE DATA, EXPRESS, IMPLIED OR ARISING\n BY CUSTOM OR TRADE USAGE, AND DISCLAIMS ANY IMPLIED WARRANTIES OF TITLE, \n NON-INFRINGEMENT, MERCHANTABILITY OR FITNESS FOR A PARTICULAR PURPOSE OR USE. \n WITHOUT LIMITING THE FOREGOING, AUTODESK DOES NOT WARRANT THAT THE OPERATION\n OF THE DATA WILL BE UNINTERRUPTED OR ERROR FREE. \n \n IN NO EVENT SHALL AUTODESK, ITS AFFILIATES, PARENT COMPANIES, LICENSORS\n OR SUPPLIERS (\"AUTODESK GROUP\") BE LIABLE FOR ANY LOSSES, DAMAGES OR EXPENSES\n OF ANY KIND (INCLUDING WITHOUT LIMITATION PUNITIVE OR MULTIPLE DAMAGES OR OTHER\n SPECIAL, DIRECT, INDIRECT, EXEMPLARY, INCIDENTAL, LOSS OF PROFITS, REVENUE\n OR DATA, COST OF COVER OR CONSEQUENTIAL LOSSES OR DAMAGES OF ANY KIND),\n HOWEVER CAUSED, AND REGARDLESS OF THE THEORY OF LIABILITY, WHETHER DERIVED\n FROM CONTRACT, TORT (INCLUDING, BUT NOT LIMITED TO, NEGLIGENCE), OR OTHERWISE,\n ARISING OUT OF OR RELATING TO THE DATA OR ITS USE OR ANY OTHER PERFORMANCE,\n WHETHER OR NOT AUTODESK HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH LOSS\n OR DAMAGE. \n\n**************************************************************************************/\n#include \"MyOwnReader.h\"\n#include \"MyOwnWriter.h\"\n\n#define PLUGIN_NAME		\"My_Own_Writer_Reader\"\n#define PLUGIN_VERSION		\"1.0\"\n#define PLUGIN_EXTENSION	\"ABC\"\n\n// Create your own writer.\n// And your writer will get a pPluginID and pSubID. \nFbxWriter* CreateMyOwnWriter(FbxManager&amp; pManager, FbxExporter&amp; pExporter, int pSubID, int pPluginID)\n{\n	FbxWriter* lWriter = FbxSdkNew&lt; MyOwnWriter &gt;(pManager, pPluginID);\n	lWriter-&gt;SetIOSettings(pExporter.GetIOSettings());\n	return lWriter;\n}\n\n// Get extension, description or version info about MyOwnWriter\nvoid* GetMyOwnWriterInfo(FbxWriter::KInfoRequest pRequest, int pId)\n{\n    static char const* sExt[] = {PLUGIN_EXTENSION, 0};\n    static char const* sDesc[] = {PLUGIN_NAME\" Writer\", 0};\n\n    switch( pRequest )\n    {\n		case FbxWriter::eInfoExtension:	return sExt;\n		case FbxWriter::eInfoDescriptions:	return sDesc;\n		case FbxWriter::eInfoVersions:		return 0;\n		default:							return 0;\n    }\n}\n\nvoid FillOwnWriterIOSettings(FbxIOSettings&amp; pIOS)\n{\n    // Here you can write your own FbxIOSettings and parse them.\n    FbxProperty FBXExtentionsSDKGroup = pIOS.GetProperty(EXP_FBX_EXT_SDK_GRP);\n    if( !FBXExtentionsSDKGroup.IsValid() ) return;\n\n    FbxProperty IOPluginGroup = pIOS.AddPropertyGroup(FBXExtentionsSDKGroup, PLUGIN_NAME, DTString, PLUGIN_NAME);\n    if( IOPluginGroup.IsValid() )\n    {\n        //Add your plugin export options here...\n        //Example:\n        bool Default_True = true;\n        pIOS.AddProperty(IOPluginGroup, \"Test\", DTBool, \"Test\", &amp;Default_True, eBOOL1);\n    }\n}\n\n\n// Creates a MyOwnReader in the Sdk Manager\nFbxReader* CreateMyOwnReader(FbxManager&amp; pManager, FbxImporter&amp; pImporter, int pSubID, int pPluginID)\n{\n	FbxReader* lReader = FbxSdkNew&lt; MyOwnReader &gt;(pManager, pPluginID);\n	lReader-&gt;SetIOSettings(pImporter.GetIOSettings());\n	return lReader;\n}\n\n// Get extension, description or version info about MyOwnReader\nvoid *GetMyOwnReaderInfo(FbxReader::KInfoRequest pRequest, int pId)\n{\n    static char const* sExt[] = {PLUGIN_EXTENSION, 0};\n    static char const* sDesc[] = {PLUGIN_NAME\" Reader\", 0};\n\n    switch( pRequest )\n    {\n		case FbxReader::eInfoExtension:	return sExt;\n		case FbxReader::eInfoDescriptions:	return sDesc;\n		default:							return 0;\n    }\n}\n\nvoid FillOwnReaderIOSettings(FbxIOSettings&amp; pIOS)\n{    \n    // Here you can write your own FbxIOSettings and parse them.\n    FbxProperty FBXExtentionsSDKGroup = pIOS.GetProperty(IMP_FBX_EXT_SDK_GRP);\n    if( !FBXExtentionsSDKGroup.IsValid() ) return;\n\n    FbxProperty IOPluginGroup = pIOS.AddPropertyGroup(FBXExtentionsSDKGroup, PLUGIN_NAME, DTString, PLUGIN_NAME);\n    if( IOPluginGroup.IsValid() )\n    {\n        //Add your plugin import options here...\n        //Example:\n        bool Default_True = true;\n        pIOS.AddProperty(IOPluginGroup, \"Test\", DTBool, \"Test\", &amp;Default_True, eBOOL1);\n    }\n}\n\n\nclass MyOwnWriterReaderPlugin : public FbxPlugin\n{\n    FBXPLUGIN_DECLARE(MyOwnWriterReaderPlugin);\n\nprotected:\n    explicit MyOwnWriterReaderPlugin(const FbxPluginDefinition&amp; pDefinition, kLibHandle pLibHandle) : FbxPlugin(pDefinition, pLibHandle)\n    {\n    }\n\n    // Implement Fbxmodules::FbxPlugin\n    virtual bool SpecificInitialize()\n    {\n        int FirstPluginID, RegistredCount;\n        GetData().mSDKManager-&gt;GetIOPluginRegistry()-&gt;RegisterReader(CreateMyOwnReader, GetMyOwnReaderInfo, FirstPluginID, RegistredCount, FillOwnReaderIOSettings);\n        GetData().mSDKManager-&gt;GetIOPluginRegistry()-&gt;RegisterWriter(CreateMyOwnWriter, GetMyOwnWriterInfo, FirstPluginID, RegistredCount, FillOwnWriterIOSettings);\n        return true;\n    }\n\n    virtual bool SpecificTerminate()\n    {\n        return true;\n    }\n};\n\nFBXPLUGIN_IMPLEMENT(MyOwnWriterReaderPlugin);\n\n// FBX Interface\nextern \"C\"\n{\n    //The DLL is owner of the plug-in\n    static MyOwnWriterReaderPlugin* sPlugin = NULL;\n\n    //This function will be called when an application will request the plug-in\n#ifdef KARCH_ENV_WIN\n	__declspec(dllexport) void FBXPluginRegistration(FbxPluginContainer&amp; pContainer, kLibHandle pLibHandle)\n#else\n	void FBXPluginRegistration(FbxPluginContainer&amp; pContainer, kLibHandle pLibHandle)\n#endif\n    {\n        if( sPlugin == NULL )\n        {\n            //Create the plug-in definition which contains the information about the plug-in\n            FbxPluginDefinition sPluginDef;\n            sPluginDef.mName = PLUGIN_NAME\" Plugin\";\n            sPluginDef.mVersion = PLUGIN_VERSION;\n\n            //Create an instance of the plug-in.  The DLL has the ownership of the plug-in\n            sPlugin = MyOwnWriterReaderPlugin::Create(sPluginDef, pLibHandle);\n\n            //Register the plug-in\n            pContainer.Register(*sPlugin);\n        }\n    }\n}\n</pre></div> \n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";