var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"FBX\">\n      <meta name=\"release\" content=\"2018\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2017-04-25\">\n      <meta name=\"topicid\" content=\"GUID-9481A726-315C-4A58-A347-8AC95C2AF0F2\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Extracting the Animation Data from a FBX File</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-9481A726-315C-4A58-A347-8AC95C2AF0F2\"></a><div class=\"head\">\n            <h1>Extracting the Animation Data from a FBX File</h1>\n         </div>\n         <p><a name=\"GUID-A31CC0F8-0D42-4F8C-8A7E-1D0712456D9F\"></a>You can extract and query the animation data from a scene (FBX file) without loading\n            the entire FBX file. \n         </p>\n         <p><a name=\"WS1A9193826455F5FF45564F421269B08A8E9-56A1\"></a>A scene (<span class=\"code\">FbxScene</span>) can contain one or more animation stacks (<span class=\"code\">FbxAnimStack</span>). The animation stack is the highest-level container for the animation data and contains\n            one or more animation layers (<span class=\"code\">FbxAnimLayer</span>). An animation layer contains one or more animation curve nodes (<span class=\"code\">FbxAnimCurveNode</span>). An animation curve node is the connection point between the animation curves (<span class=\"code\">FbxAnimCurve</span>) and the FBX properties (<span class=\"code\">FbxProperty</span>) of a FBX object (<span class=\"code\">FbxObject</span>). \n         </p><a name=\"OL_7AD28B40463F45D6AFBE54B05186C530\"></a><p class=\"list-intro\"><a name=\"GUID-6FCFF991-95A3-4C1C-BE4C-37C26B52CB27\"></a>Perform the following steps to extract the animation data from a scene:\n         </p>\n         <ol type=\"1\" start=\"1\">\n            <li>Extract the animation stacks using a pointer to an instance of the FbxScene (pScene).\n               \n               <div class=\"codeBlock\"><pre>int numStacks = pScene-&gt;GetSrcObjectCount(FBX_TYPE(FbxAnimStack)); i++)</pre></div> \n            </li>\n            <li>Retrieve the nth animation stack. \n               <div class=\"codeBlock\"><pre>FbxAnimStack* pAnimStack = FbxCast&lt;FbxAnimStack&gt;(pScene-&gt;GetSrcObject(FBX_TYPE(FbxAnimStack), n)</pre></div> \n            </li>\n            <li>Retrieve the number of animation layers in an animation stack. \n               <div class=\"codeBlock\"><pre>int numAnimLayers = pAnimStack-&gt;GetMemberCount(FBX_TYPE(FbxAnimLayer));</pre></div> \n            </li>\n            <li>Retrieve the nth animation layer from the animation stack. \n               <div class=\"codeBlock\"><pre>FbxAnimLayer* lAnimLayer = lAnimStack-&gt;GetMember(FBX_TYPE(FbxAnimLayer), n);</pre></div> \n               <p><a name=\"GUID-DDD06925-A505-416F-9346-361B2242D5C0\"></a>After retrieving an animation layer, you can access the animation curves for the properties\n                  of nodes and node attribute. \n               </p> \n            </li>\n            <li>Retrieve the animation curves. <a name=\"UL_B4EC03D00C504259B1A53DD5321B64BC\"></a><ul>\n                  <li>Given a node (lNode) and an animation layer (lAnimLayer), you can retrieve the animation\n                     curves for node properties such as the local translation as shown in the following\n                     example. \n                     <div class=\"codeBlock\"><pre>FbxAnimCurve* lAnimCurve = lNode-&gt;LclTranslation.GetCurve(lAnimLayer, FBXSDK_CURVENODE_COMPONENT_X);</pre></div> \n                  </li>\n                  <li>Given a node (lNode) and a layer (lAnimLayer), you can retrieve the animation curves\n                     for node attribute properties such as the red color component as shown in the following\n                     example. \n                     <div class=\"codeBlock\"><pre>FbxNodeAttribute* lNodeAttribute = lNode-&gt;GetNodeAttribute();\nFbxAnimCurve* lAnimCurve = lNodeAttribute-&gt;Color.GetCurve&lt;FbxAnimCurve&gt;(pAnimLayer, FBXSDK_CURVENODE_COLOR_RED);</pre></div> \n                  </li>\n               </ul> \n            </li>\n         </ol>\n         <p><a name=\"GUID-53E9564B-1A9C-4354-BC82-B508F58CECFC\"></a>You can also use the animation curve nodes to access the animation data for the properties.\n            The following steps show how to access the animation curves for a specific property\n            on the first (0th) channel. \n         </p><a name=\"OL_7BEB2B8A783C4F0E99807081F7A91072\"></a><ol type=\"1\" start=\"1\">\n            <li>Check whether the property is valid. \n               <div class=\"codeBlock\"><pre>if (lProperty.IsValid) ...</pre></div> \n            </li>\n            <li>Retrieve the curve node using the animation layer. \n               <div class=\"codeBlock\"><pre>FbxAnimCurveNode* lCurveNode = lProperty.GetCurveNode(pAnimLayer);</pre></div> \n               <p><a name=\"GUID-390E2180-F1C8-4CB2-BE76-311A7DDFFE45\"></a>If the property is not animated the curve node is NULL. \n                  <div class=\"codeBlock\"><pre>if (lCurveNode != NULL) ...</pre></div> \n               </p> \n            </li>\n            <li>Step through the curves in the curve node. \n               <div class=\"codeBlock\"><pre>for( int c = 0; c &lt; lCurveNode-&gt;GetCurveCount(0U); c++ ) {          \n                                FbxAnimCurve* lAnimCurve = lCurveNode-&gt;GetCurve(0U, c);\n                                if (lAnimCurve != NULL) {\n                                                // ...\n                                }\n                }\n</pre></div> \n            </li>\n         </ol>\n         <p><a name=\"GUID-7BE201C5-838A-49D2-975A-2118A9436E1C\"></a>See the <span class=\"code\">FbxAnimCurveNode</span> class reference for more information on animation curve nodes and channels. \n         </p>\n         <p><a name=\"GUID-418BF936-9147-4C53-BE15-CEF6CF4B7E6E\"></a>For a more detailed example of importing animation data using the FBX SDK, see the\n            example <span class=\"code\">ImportScene/DisplayAnimation.cxx</span>. \n         </p>\n         <div class=\"related-links\">\n            <h4 class=\"related\">Related Links</h4>\n            <ul>\n               <li><span class=\"char_link\"><a href=\"GUID-ABFC58F1-C10D-42AE-A204-D79EB1D84B24.htm\">Importing a Scene </a></span></li>\n               <li><span class=\"char_link\"><a href=\"GUID-386BAD6A-82F9-4148-BD5F-3FE2E90A87D0.htm\">Animation classes and their interrelationships </a></span></li>\n               <li><span class=\"char_link\"><a href=\"GUID-3DC64450-58B7-45D6-B7DD-F18190C247DB.htm\">Migrating to the new data structures for animation </a></span></li>\n            </ul>\n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";