var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"FBX\">\n      <meta name=\"release\" content=\"2018\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2017-04-25\">\n      <meta name=\"topicid\" content=\"GUID-ABFC58F1-C10D-42AE-A204-D79EB1D84B24\">\n      <meta name=\"indexterm\" content=\"file importer, creating\">\n      <meta name=\"indexterm\" content=\"importer, creating file\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Importing a Scene</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-ABFC58F1-C10D-42AE-A204-D79EB1D84B24\"></a><div class=\"head\">\n            <h1>Importing a Scene</h1>\n         </div>\n         <p><a name=\"GUID-12C0AEB6-6B8E-441C-AF98-11D87EC82208\"></a>A scene can be imported by the FBX SDK from various file formats. The FBX file format\n            can either be in <em class=\"mild\">binary</em> (also known as the <em class=\"mild\">native</em> format), or in ASCII. Note that only the binary FBX file format supports embedded\n            media (see <span class=\"char_link\"><a href=\"GUID-50489A8A-457C-4B54-80E1-5572A16F7F17.htm\">I/O Settings</a></span>). The Collada file format and other non-FBX file formats can also be used to import\n            scenes. For more information, see <span class=\"char_link\"><a href=\"GUID-0B122E01-7DB8-48E3-AADA-5E85A197FEE1.htm\">Supported File Formats</a></span>. \n         </p>\n         <div class=\"section\"><a name=\"SECTION_1E9D7628A63E433F8ADCC4451D01DFDA\"></a> \n            <h2><a name=\"GUID-57883336-7077-479A-B7E6-147DD26A9830\"></a>Initializing the Importer\n            </h2> \n            <p><a name=\"GUID-D8550E65-F3CD-4704-9039-FB961F7E9E48\"></a>The scene importing functionality of the FBX SDK is abstracted by the <span class=\"code\">FbxImporter</span> class. Instances of <span class=\"code\">FbxImporter</span> are created with a reference to the program\'s <span class=\"code\">FbxManager</span> singleton object. A <span class=\"code\">FbxImporter</span> must have its <span class=\"code\">FbxImporter::Initialize()</span> method called with three parameters: <a name=\"OL_F12980BA833240E4AA731FF219DB3389\"></a><ol type=\"1\" start=\"1\">\n                  <li>The path and filename of the file containing the scene to import. </li>\n                  <li>The numeric file format identifier. Typically, this parameter is set to <span class=\"code\">-1</span> to let the importer automatically detect the file format according to the provided\n                     filename\'s extension. \n                  </li>\n                  <li>The <span class=\"code\">FbxIOSettings</span> object containing the import configuration options. See <span class=\"char_link\"><a href=\"GUID-50489A8A-457C-4B54-80E1-5572A16F7F17.htm\">I/O Settings</a></span> for more information. \n                  </li>\n               </ol> \n            </p> \n            <div class=\"codeBlock\"><pre>#include &lt;fbxsdk.h&gt;\n#include &lt;fbxfilesdk/fbxio/fbxiosettings.h&gt;\n\n// ...\n\n// Create the FBX SDK manager\nFbxManager* lSdkManager = FbxManager::Create();\n\n// Create an IOSettings object.\nFbxIOSettings * ios = FbxIOSettings::Create(lSdkManager, IOSROOT );\nlSdkManager-&gt;SetIOSettings(ios);\n\n// ... Configure the FbxIOSettings object ...\n\n// Create an importer.\nFbxImporter* lImporter = FbxImporter::Create(lSdkManager, \"\");\n\n// Declare the path and filename of the file containing the scene.\n// In this case, we are assuming the file is in the same directory as the executable.\nconst char* lFilename = \"file.fbx\";\n\n// Initialize the importer.\nbool lImportStatus = lImporter-&gt;Initialize(lFilename, -1, lSdkManager-&gt;GetIOSettings());</pre></div> \n            <p><a name=\"GUID-95DBFC88-436B-403C-988D-B83D5E0A9094\"></a>If any errors occur in the call to <span class=\"code\">FbxImporter::Initialize()</span>, the method returns <span class=\"code\">false</span>. To retrieve the error, you must call <span class=\"code\">GetStatus().GetErrorString()</span> from the <span class=\"code\">FbxImporter</span> object. For more information on error handling, see <span class=\"char_link\"><a href=\"GUID-5509751F-8679-4D32-987D-8343FD8F6D1A.htm\">Error Handling</a></span>. \n            </p> \n            <div class=\"codeBlock\"><pre>if(!lImportStatus) {\n    printf(\"Call to FbxImporter::Initialize() failed.\\n\");\n    printf(\"Error returned: %s\\n\\n\", lImporter-&gt;GetStatus().GetErrorString());\n    exit(-1);\n}</pre></div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_FBFC7D23AB7E438DB760B189C65C371F\"></a> \n            <h2><a name=\"GUID-943578F5-E9EF-4A8D-BD4F-4F19E74DA982\"></a>Importing a Scene\n            </h2> \n            <p><a name=\"GUID-DCD657B0-9D14-4ABF-9045-424D5DAEA017\"></a>Once the importer has been initialized, a scene container must be created to load\n               the scene from the file. Scenes in the FBX SDK are abstracted by the <span class=\"code\">FbxScene</span> class. For more information on scenes, see <span class=\"char_link\"><a href=\"GUID-F194000D-5AD4-49C1-86CC-5DAC2CE64E97.htm\">Nodes and the Scene Graph</a></span>. \n            </p> \n            <div class=\"codeBlock\"><pre>// Create a new scene so it can be populated by the imported file.\nFbxScene* lScene = FbxScene::Create(lSdkManager,\"myScene\");\n\n// Import the contents of the file into the scene.\nlImporter-&gt;Import(lScene);</pre></div> \n            <p><a name=\"GUID-68CA4D4A-B641-4006-9C37-0FBF96A6E03F\"></a>After the importer has populated the scene, it is safe to destroy it to reduce memory\n               usage. \n            </p> \n            <div class=\"codeBlock\"><pre>// The file has been imported; we can get rid of the importer.\nlImporter-&gt;Destroy();</pre></div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_79BF9326ABD6459CB4265186C2F98CE6\"></a> \n            <h2><a name=\"GUID-EDCA2BB3-A765-4AB1-A2DC-BF208E440EBB\"></a>File Version Number\n            </h2> \n            <p><a name=\"GUID-A92F382D-B774-4406-A02E-FBCEC0EADD47\"></a>The FBX file format version is incremented to reflect newly supported features (see\n               <span class=\"char_link\"><a href=\"GUID-0B122E01-7DB8-48E3-AADA-5E85A197FEE1.htm\">Supported File Formats</a></span>). The FBX version of the currently imported file can be obtained by calling <span class=\"code\">FbxImporter::GetFileVersion()</span>. \n            </p> \n            <div class=\"codeBlock\"><pre>// File format version numbers to be populated.\nint lFileMajor, lFileMinor, lFileRevision;\n\n// Populate the FBX file format version numbers with the import file.\nlImporter-&gt;GetFileVersion(lFileMajor, lFileMinor, lFileRevision);</pre></div> \n            <p><a name=\"GUID-8C46FD70-D561-468D-9B28-9C54FC6E3DC5\"></a>As of FBX SDK 2012.0, the header file <span class=\"filePath\">fbxfilesdk_version.h</span> defines the preprocessor identifer <span class=\"code\">FBXSDK_VERSION_STRING</span>, which represents the version information as a string. \n            </p> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_C6B336F1DF204C4299E8A9675C7C4D6C\"></a> \n            <h2><a name=\"GUID-0FC5C9E3-EEBF-407E-93D1-3ECAA76A1776\"></a>Imported Custom Objects\n            </h2> \n            <p><a name=\"GUID-A79F4242-58FC-499E-B12B-760F7B3B2B77\"></a>When the FBX SDK encounters a new object that it does not recognize during the importing\n               process, it creates a new class for it at runtime. To access this object, add a filter\n               to the <span class=\"code\">FbxObject::GetSrcObjectCount()</span> and <span class=\"code\">FbxObject::GetSrcObject()</span> functions. The concept of sources, destinations, and connections in the context of\n               the FBX SDK is explained in <span class=\"char_link\"><a href=\"GUID-E802CA6C-4608-41ED-A35A-017CE6E9E8DD.htm\">FBX SDK Object Model</a></span>. \n            </p> \n            <div class=\"codeBlock\"><pre>FbxClassId lShaderClassID = lSdkManager-&gt;FindFbxFileClass(\"Shader\", \"FlatShader\");\nfor(int i = 0; i &lt; lNode-&gt;GetSrcObjectCount(lShaderClassID); i++) {\n  FbxObject* lObject = lNode-&gt;GetSrcObject(lShaderClassID, i);\n}</pre></div> \n         </div>\n         <div class=\"see-also\">\n            <h4>See Also</h4>\n            <div class=\"related-links\">\n               <ul>\n                  <li><span class=\"char_link\"><a href=\"GUID-75CD0DC4-05C8-4497-AC6E-EA11406EAE26.htm\">Customizing File Formats with FBX SDK I/O Plug-ins </a></span></li>\n               </ul>\n            </div>\n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";