var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"FBX\">\n      <meta name=\"release\" content=\"2018\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2017-04-25\">\n      <meta name=\"topicid\" content=\"GUID-C08BAE05-B074-441C-9DBE-CE1811C0E6E2\">\n      <meta name=\"description\" content=\"This topic presents how to merge two imported scenes. It also provides basic insight on manipulating a scene and the nodes within it, and will initiate the reader to the concept of connections and connection management.\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Merging Two Scenes</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-C08BAE05-B074-441C-9DBE-CE1811C0E6E2\"></a><div class=\"head\">\n            <h1> Merging Two Scenes </h1>\n         </div>\n         <p class=\"blurb\">This topic presents how to merge two imported scenes. It also provides basic insight\n            on manipulating a scene and the nodes within it, and will initiate the reader to the\n            concept of connections and connection management. \n            \n         </p><a name=\"OL_7928F8DB721949398E1BFF08692DBCFC\"></a><p class=\"list-intro\"><a name=\"GUID-9DD92D0D-EB0D-4727-8270-F4E538F283EE\"></a>The steps taken for merging two scenes are as follows:\n         </p>\n         <ol type=\"1\" start=\"1\">\n            <li>Load a scene. \n               		\n            </li>\n            <li>Modify a node within the loaded scene. \n               		\n            </li>\n            <li>Move the contents of the loaded scene into a reference scene. \n               		\n            </li>\n            <li>Load a different scene. \n               		\n            </li>\n            <li>Update the newly loaded scene with the contents of the reference scene. \n               		\n            </li>\n            <li>Destroy the reference scene. \n               		\n            </li>\n         </ol>\n         <div class=\"section\"><a name=\"SECTION_E3B140B542F64F9F8802B963B073C3D8\"></a> \n            <h2><a name=\"GUID-0472C265-5A6A-47DB-9606-B5B1D6CCCF96\"></a>Loading and modifying the scene\n            </h2> \n            <p><a name=\"GUID-10B622FF-DA3F-452C-B265-64D867C2BFC4\"></a>The first object we create is the \n               		  <span class=\"code\">FbxManager</span>; this step is required to create subsequent FBX SDK objects. A newly created \n               		  <span class=\"code\">FbxScene</span> object, referred to as the \"current scene\", is then passed to our custom \n               		  <span class=\"code\">LoadScene()</span> function, so it can be populated by the imported contents of \"file1.fbx\" (which should\n               be in the same folder as the executable program). Once the scene\'s contents are populated,\n               we simply change the name of the root node\'s first child (indexed at 0) to \"Test Name\".\n               \n               		\n            </p> \n            <div class=\"codeBlock\"><pre>    // Create an SDK manager.\n    FbxManager* lSdkManager = FbxManager::Create();\n\n    // Create a new scene so it can be populated by the imported file.\n    FbxScene* lCurrentScene = FbxScene::Create(lSdkManager,\"My Scene\");\n\n    // Load the scene.\n    LoadScene(lSdkManager, lCurrentScene, \"file1.fbx\");\n\n    // Modify the scene. In this example, only one node name is changed.\n    lCurrentScene-&gt;GetRootNode()-&gt;GetChild(0)-&gt;SetName(\"Test Name\");</pre></div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_E51FDCB8A34444E9932D55C25224E3E7\"></a> \n            <h2><a name=\"GUID-7110E7D5-A366-4C3A-9884-74E4539B9F74\"></a>Moving the contents of the loaded scene into a reference scene\n            </h2> \n            <p><a name=\"GUID-5CAD2E43-726F-411E-B013-B905E63F790C\"></a>Another \n               		  <span class=\"code\">FbxScene</span> object is created, which we will refer to as the \"reference scene\". Note that when\n               a \n               		  <span class=\"code\">FbxScene</span> object is created, its root node and its global settings are also created with it.\n               As such, the objective of this step is to remove the children from the current scene\'s\n               root node so they may become the children of the reference scene\'s root node. \n               		\n            </p> \n            <div class=\"codeBlock\"><pre>    // Create a reference scene to store the contents of the currently loaded scene.\n    FbxScene *lMyRefScene = FbxScene::Create(lSdkManager, \"My Reference Scene\");\n\n    // Move the node tree of the currently loaded scene into the reference scene.\n    vector&lt;FbxNode*&gt; lChildren;\n    int lNumChildren = lCurrentScene-&gt;GetRootNode()-&gt;GetChildCount();\n    for(int i = 0; i &lt; lNumChildren; i++) {\n\n        // Obtain a child node from the currently loaded scene.\n        lChildren.push_back(lCurrentScene-&gt;GetRootNode()-&gt;GetChild(i));\n\n        // Attach the child node to the reference scene\'s root node.\n       for( int c = 0; c &lt; lChildren.size(); c )\n           lMyRefScene-&gt;GetRootNode()-&gt;AddChild(lChildren[c]);\n    }\n\n    // Remove the children from the root node.\n    lCurrentScene-&gt;GetRootNode()-&gt;DisconnectAllSrcObject();</pre></div> \n            <p><a name=\"GUID-AC099689-B0D6-4AE0-AEC8-A5D3C93C660B\"></a>Other \n               		  <span class=\"code\">FbxObjects</span> contained in the current scene may need to be moved to the reference scene. Such\n               objects may include instances of: \n               		  <span class=\"code\">FbxCharacter</span>, \n               		  <span class=\"code\">FbxCharacterPose</span>, and \n               		  <span class=\"code\">FbxDocumentInfo</span> to name a few. Observe that these objects are not children of the current scene\'s\n               root node, and need to be moved differently. To achieve this, we will require some\n               basic knowledge of \"<span class=\"char_link\"><a href=\"GUID-BB63A93A-7663-4256-B060-8EA35CB0FF3A.htm\">connections</a></span>\" in the context of the FBX SDK. \n               		\n            </p> \n            <p><a name=\"GUID-CBCB4737-3F33-4051-B22C-303FB5BB6D8F\"></a>A connection is a directional \"source to destination\" relationship between either:\n               two \n               		  <span class=\"code\">FbxObject</span>s, a \n               		  <span class=\"code\">FbxObject</span> and a \n               		  <span class=\"code\">FbxProperty</span>, or two \n               		  <span class=\"code\">FbxProperty</span>s. For more information on connections, see \n               		  <span class=\"char_link\"><a href=\"GUID-BB63A93A-7663-4256-B060-8EA35CB0FF3A.htm\">Connections</a></span>. In the scope of this tutorial, we want to disconnect all the relevant source objects\n               from the current scene, and connect them the reference scene. We do this using the\n               \n               		  <span class=\"code\">FbxObject::GetSrcObjectCount()</span>, \n               		  <span class=\"code\">FbxObject::DisconnectAllSrcObject()</span>, and \n               		  <span class=\"code\">FbxObject::ConnectDstObject()</span>. \n               		\n            </p> \n            <div class=\"codeBlock\"><pre>    // Move other objects to the reference scene.\n    int lNumSceneObjects = lCurrentScene-&gt;GetSrcObjectCount();\n    for(int i = 0; i &lt; lNumSceneObjects; i++) {\n        FbxObject* lObj = lCurrentScene-&gt;GetSrcObject(i);\n        if(lObj == lCurrentScene-&gt;GetRootNode() || *lObj == lCurrentScene-&gt;GetGlobalSettings()){\n            // Don\'t move the root node or the scene\'s global settings; these\n            // objects are created for every scene.\n            continue;\n        }\n        // Attach the object to the reference scene.\n        lObj-&gt;ConnectDstObject(lMyRefScene);\n    }\n\n    // Disconnect all scene objects.\n    lCurrentScene-&gt;DisconnectAllSrcObject();</pre></div> \n            <div><a name=\"GUID-266594DE-5607-4B44-95B7-762214ACF782\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span>You can get all the objects of a certain type in a scene using the concept of connections.\n                  In the following code snippet, we iterate through the meshes connected to a scene\n                  by calling \n                  		  <span class=\"code\">lScene-&gt;GetSrcObjectCount(FbxMesh::ClassId)</span>. \n                  		  \n                  <div class=\"codeBlock\"><pre>for(int i = 0; i &lt; lScene-&gt;GetSrcObjectCount(FbxMesh::ClassId); i++) {\n    FbxMesh* lMesh = (FbxMesh*)lScene-&gt;GetSrcObject(FbxMesh::ClassId, i);\n    //...\n}</pre></div> \n               </div>\n            </div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_3A97B32444934014897CBB7CA8BE4EB5\"></a> \n            <h2><a name=\"GUID-82749416-5BFA-4956-B10E-CC7D8A56CE2D\"></a>Loading and updating a different scene\n            </h2> \n            <p><a name=\"GUID-8150C8AC-A5A2-47AF-AAE5-2B87A766C776\"></a>At this point, we have moved the elements of the current scene into the reference\n               scene, and the current scene has become empty. We now take our current scene and repopulate\n               it with the contents of \"file2.fbx\". Finally, we change the name of the root\'s first\n               child in the current scene to the name of root\'s first child in the reference scene.\n               In the end, the name of the root\'s first child in the current scene should be \"Test\n               Name\". \n               		\n            </p> \n            <div class=\"codeBlock\"><pre>    // Import the second file into lCurrentScene.\n    LoadScene(lSdkManager, lCurrentScene, \"file2.fbx\");\n\n    // Get the names after the second file is loaded.\n    FbxString lNameBeforeUpdate = lCurrentScene-&gt;GetRootNode()-&gt;GetChild(0)-&gt;GetName();\n    FbxString lReferenceName = lMyRefScene-&gt;GetRootNode()-&gt;GetChild(0)-&gt;GetName();\n\n    // Update the root\'s child 0 name.\n    lCurrentScene-&gt;GetRootNode()-&gt;GetChild(0)-&gt;SetName(lReferenceName);\n    FbxString lNameAfterUpdate = lCurrentScene-&gt;GetRootNode()-&gt;GetChild(0)-&gt;GetName();\n\n    // Destroy the reference scene.\n    lMyRefScene-&gt;Destroy();\n    lMyRefScene = NULL;\n\n    // Verification step\n    printf(\"Verification (0 for success): %d\\n\", lNameAfterUpdate.Compare(\"Test Name\"));\n</pre></div> \n         </div>\n         <div class=\"section\"><a name=\"SECTION_31DC278132604F51AD72C23424F139F7\"></a> \n            <h2><a name=\"GUID-A8578D90-8E72-4C78-A557-F2C8E05A2511\"></a>Scene Merging Tutorial Program\n            </h2> \n            <p><a name=\"GUID-E91FFDC0-E297-4594-B57F-F4EA1612D279\"></a>The following is the scene merging tutorial program discussed above. \n               		\n            </p> \n            <div class=\"codeBlock\"><pre>#include &lt;fbxsdk.h&gt;\n\n/**\n * Load a scene given an FbxManager, a FbxScene, and a valid filename.\n */\nint LoadScene(FbxManager* pSdkManager, FbxScene* pScene, char* filename) {\n    // Create the io settings object.\n    FbxIOSettings *ios = FbxIOSettings::Create(pSdkManager, IOSROOT);\n    pSdkManager-&gt;SetIOSettings(ios);\n\n    // Create an importer using our sdk manager.\n    FbxImporter* lImporter = FbxImporter::Create(pSdkManager,\"\");\n    \n    // Use the first argument as the filename for the importer.\n    if(!lImporter-&gt;Initialize(filename, -1, pSdkManager-&gt;GetIOSettings())) {\n        printf(\"Call to FbxImporter::Initialize() failed.\\n\");\n        printf(\"Error returned: %s\\n\\n\", lImporter-&gt;GetStatus().GetErrorString());\n        lImporter-&gt;Destroy();\n        return -1;\n    }\n\n    // Import the contents of the file into the scene.\n    lImporter-&gt;Import(pScene);\n\n    // The file has been imported; we can get rid of the importer.\n    lImporter-&gt;Destroy();\n    return 0;\n}\n\n/**\n * Entry point for the merging two scenes sample program.\n */\nint main(int argc, char** argv) {\n    // Create an SDK manager.\n    FbxManager* lSdkManager = FbxManager::Create();\n\n    // Create a new scene so it can be populated by the imported file.\n    FbxScene* lCurrentScene = FbxScene::Create(lSdkManager,\"My Scene\");\n\n    // Load the scene.\n    LoadScene(lSdkManager, lCurrentScene, \"file1.fbx\");\n\n    // Modify the scene. In this example, only one node name is changed.\n    lCurrentScene-&gt;GetRootNode()-&gt;GetChild(0)-&gt;SetName(\"Test Name\");\n\n    // Create a reference scene to store the contents of the currently loaded scene.\n    FbxScene *lMyRefScene = FbxScene::Create(lSdkManager, \"My Reference Scene\");\n\n    // Move the node tree of the currently loaded scene into the reference scene.\n    vector&lt;FbxNode*&gt; lChildren;\n    int lNumChildren = lCurrentScene-&gt;GetRootNode()-&gt;GetChildCount();\n    for(int i = 0; i &lt; lNumChildren; i++) {\n\n        // Obtain a child node from the currently loaded scene.\n        lChildren.push_back(lCurrentScene-&gt;GetRootNode()-&gt;GetChild(i));\n\n        // Attach the child node to the reference scene\'s root node.\n        for( int c = 0; c &lt; lChildren.size(); c )\n           lMyRefScene-&gt;GetRootNode()-&gt;AddChild(lChildren[c]);\n    }\n\n    // Remove the children from the root node.\n    lCurrentScene-&gt;GetRootNode()-&gt;DisconnectAllSrcObject();\n\n    // Move other objects to the reference scene.\n    int lNumSceneObjects = lCurrentScene-&gt;GetSrcObjectCount();\n    for(int i = 0; i &lt; lNumSceneObjects; i++) {\n        FbxObject* lObj = lCurrentScene-&gt;GetSrcObject(i);\n        if(lObj == lCurrentScene-&gt;GetRootNode() || *lObj == lCurrentScene-&gt;GetGlobalSettings()){\n            // Don\'t move the root node or the scene\'s global settings; these\n            // objects are created for every scene.\n            continue;\n        }\n        // Attach the object to the reference scene.\n        lObj-&gt;ConnectDstObject(lMyRefScene);\n    }\n\n    // Disconnect all scene objects.\n    lCurrentScene-&gt;DisconnectAllSrcObject();\n\n    // Import the second file into lCurrentScene.\n    LoadScene(lSdkManager, lCurrentScene, \"file1.fbx\");\n\n    // Get the names after the second file is loaded.\n    FbxString lNameBeforeUpdate = lCurrentScene-&gt;GetRootNode()-&gt;GetChild(0)-&gt;GetName();\n    FbxString lReferenceName = lMyRefScene-&gt;GetRootNode()-&gt;GetChild(0)-&gt;GetName();\n\n    // Update the root\'s child 0 name.\n    lCurrentScene-&gt;GetRootNode()-&gt;GetChild(0)-&gt;SetName(lReferenceName);\n    FbxString lNameAfterUpdate = lCurrentScene-&gt;GetRootNode()-&gt;GetChild(0)-&gt;GetName();\n\n    // Destroy the reference scene.\n    lMyRefScene-&gt;Destroy();\n    lMyRefScene = NULL;\n\n    // Verification step\n    printf(\"Verification (0 for success): %d\\n\", lNameAfterUpdate.Compare(\"Test Name\"));\n\n    // Destroy the sdk manager.\n    lSdkManager-&gt;Destroy();\n    exit(0);\n}</pre></div> \n         </div>\n         <div class=\"see-also\">\n            <h4>See Also</h4>\n            <div class=\"related-links\">\n               <ul>\n                  <li><span class=\"char_link\"><a href=\"GUID-29C09995-47A9-4B49-9535-2F6BDC5C4107.htm\">Your First FBX SDK Program \n                           		</a></span></li>\n                  <li><span class=\"char_link\"><a href=\"GUID-5D8BFE45-723E-4BD9-846B-E2B2540157C9.htm\">FBX Objects \n                           		</a></span></li>\n                  <li><span class=\"char_link\"><a href=\"GUID-283ED424-E506-4A29-848F-9EAC8F1C4AF9.htm\">FBX Properties \n                           		</a></span></li>\n                  <li><span class=\"char_link\"><a href=\"GUID-BB63A93A-7663-4256-B060-8EA35CB0FF3A.htm\">Connections \n                           		</a></span></li>\n               </ul>\n            </div>\n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";