var topic = "<!DOCTYPE html>\r\n\n<!-- saved from url=(0024)http://docs.autodesk.com -->\r\n<html>\n   <head>\n      <meta http-equiv=\"Content-Type\" content=\"text/html; charset=UTF-8\">\n      <meta name=\"product\" content=\"FBX\">\n      <meta name=\"release\" content=\"2018\">\n      <meta name=\"book\" content=\"Developer\">\n      <meta name=\"created\" content=\"2017-04-25\">\n      <meta name=\"topicid\" content=\"GUID-CC93340E-C4A1-49EE-B048-E898F856CFBF\">\n      <meta name=\"topic-type\" content=\"concept\">\n      <title>Scene Axis and Unit Conversion</title><script language=\"javascript\">var index = \'index.html\';</script></head>\n   <body height=\"100%\">\n      <div class=\"body_content\" id=\"body-content\" dir=\"ltr\"><a name=\"GUID-CC93340E-C4A1-49EE-B048-E898F856CFBF\"></a><div class=\"head\">\n            <h1>Scene Axis and Unit Conversion</h1>\n         </div>\n         <div class=\"section\"><a name=\"SECTION_81B90FD4335A44D194B562E7F6236AC2\"></a> \n            <h2><a name=\"GUID-DC1A3ECC-CE54-4265-A07A-3C35E1954EA4\"></a>Scene Axis and Unit Conversion\n            </h2> \n            <p><a name=\"GUID-D7998CB2-E49C-4802-8ECD-58460172A66A\"></a>Objects in the FBX SDK are always created in the \n               		  <em class=\"strong\">right handed, Y-Up axis system</em>. The scene\'s axis system may need to be converted to suit your application\'s needs.\n               Consult the \n               		  <span class=\"code\">FbxAxisSystem</span> class documentation for more information. \n               		\n            </p> \n            <p><a name=\"GUID-BD345428-B185-4964-8A0E-4D2F5F1F2494\"></a> <a name=\"UL_B30E6F98A30143DAAB2FF092B44D75A0\"></a><p class=\"list-intro\"><a name=\"GUID-80DF94D5-D347-4E91-B895-3C5190E8C4A7\"></a> Important points about the FBX SDK system unit:\n               </p>\n               <ul>\n                  <li> If a reading application needs to determine the system unit in which the data was\n                     created, then the original system unit is used as reference. \n                     		  \n                  </li>\n                  <li> A writing application must explicitly set the system unit. \n                     		  \n                  </li>\n                  <li> The implicit coordinate system is in \n                     			 <em class=\"mild\">centimeters</em>. It means that a plane with coordinates (0,0,0), (1,0,0), (1,1,0), (0,1,0) and all\n                     scaling/unit factors equal to one defines a \"1x1 cm\" plane. \n                     		  \n                  </li>\n               </ul> \n            </p> \n            <p><a name=\"GUID-FF60D5FA-AF30-425E-8BC0-DBB6E397A852\"></a> <a name=\"UL_5DD397BDF813477FBA0C2D350C9F1FDC\"></a><p class=\"list-intro\"><a name=\"GUID-88A76533-1E72-40A7-B849-44116C866749\"></a>A scene\'s axis and unit system can be respectively changed using the following functions:\n               </p>\n               <ul>\n                  <li><span class=\"code\">FbxAxisSystem::ConvertScene()</span>, which acts on the node transforms (pre-rotation) and animations. \n                     		  \n                  </li>\n                  <li><span class=\"code\">FbxSystemUnit::ConvertScene()</span>, which acts on the node transforms (scale) and animations. \n                     		  \n                  </li>\n               </ul> \n            </p> \n            <p><a name=\"GUID-E0D24A82-094A-4BF4-97F6-CFCCD0B93872\"></a>Note that calls to \n               		  <span class=\"code\">ConvertScene()</span> do not change the vertex values of meshes, and only affect node transforms and animations.\n               If the scene is already in the required axis system or required unit system, a call\n               to \n               		  <span class=\"code\">ConvertScene()</span> will have no effect on the scene. For example: \n               		<a name=\"UL_2521297EE94D4E8490197326966D204A\"></a><ul>\n                  <li>If the scene is already in the MayaZUp axis system, the code \n                     			 <span class=\"code\">FbxAxisSystem::MayaZUp.ConvertScene(scene);</span> will not change anything. \n                     		  \n                  </li>\n                  <li>If the scene is already in centimeters, the code \n                     			 <span class=\"code\">FbxSystemUnit::cm.ConvertScene(scene);</span> will not change anything. \n                     		  \n                  </li>\n               </ul> \n            </p> \n            <p><a name=\"GUID-D86D762D-B238-41CF-866E-C90B6CCDAA84\"></a> \n               <div><a name=\"GUID-E9CB216D-998F-45D5-98F9-BA812B56F1B3\"></a><div class=\"note-note\"><span class=\"label label-target-language\">NOTE:</span>If your scene is improperly scaled after unit conversion, this might be caused by\n                     different nodes inherit types (<span class=\"code\">ETransformInheritType</span>), specifically the nodes with inherit type \n                     			 <span class=\"code\">eINHERIT_Rrs</span>. To avoid this problem, make sure to avoid unit conversion on these nodes by using\n                     the conversion options in the following code snippet. This code snippet additionally\n                     illustrates how to convert a scene\'s units from centimeters (cm) into meters (m):\n                     \n                     			 \n                     <div class=\"codeBlock\"><pre>if(lScene-&gt;GetGlobalSettings().GetSystemUnit() == FbxSystemUnit::cm)\n{\n  const FbxSystemUnit::ConversionOptions lConversionOptions = {\n    false, /* mConvertRrsNodes */\n    true, /* mConvertLimits */\n    true, /* mConvertClusters */\n    true, /* mConvertLightIntensity */\n    true, /* mConvertPhotometricLProperties */\n    true  /* mConvertCameraClipPlanes */\n  };\n  \n  // Convert the scene to meters using the defined options.\n  FbxSystemUnit::m.ConvertScene(lScene, lConversionOptions);\n}</pre></div> \n                  </div>\n               </div> \n            </p> \n         </div>\n         <div class=\"footer-block\"><a href=\"..\" class=\"comments-anchor\" target=\"_blank\"><span class=\"comments-link\">Please send us your comment about this page</span></a></div><br><p class=\"footer-license-block\"><a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\"><img alt=\"Creative Commons License\" style=\"border-width: 0;\" src=\"../images/ccLink.png\"></a>&nbsp;<em>Except where otherwise noted, this work is licensed under a <a rel=\"license\" href=\"http://creativecommons.org/licenses/by-nc-sa/3.0/\" target=\"_blank\">Creative Commons Attribution-NonCommercial-ShareAlike 3.0 Unported License</a>. Please see the <a href=\"http://autodesk.com/creativecommons\" target=\"_blank\">Autodesk Creative Commons FAQ</a> for more information.</em></p><br></div>\n   </body>\n</html>";